/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;
using System.IO;
using System.Text;
using MBovin.TeamStats.Core;
using MBovin.TeamStats.Util;

namespace MBovin.TeamStats
{
	/// <summary>
	/// Holds the state of the application. The AppState is implemented
	/// as a singleton, use GetInstance() to get the AppState instance.
	/// </summary>
	public class AppState
	{
        // Constants
        private const string cSettingsFilename = "settings.dat";
        private const int cFileVersion         = 2;
        private const int cLeagueNameChars     = 16;
        private const int cSeasonChars         = 7;

        // The singleton instance
        private static AppState sInstance = new AppState();

        private string   mCurrentFilename;
        private string   mCurrentLeagueName;
        private string   mCurrentSeason;
        private string[] mLeagueNames        = new string[0];
        private IList    mDownloadLeagueList = new ArrayList();
        private string   mLiveUpdateLog;

        private LeagueViewMode mMode = LeagueViewMode.Table;
		
        
        /// <summary>
        /// Private constructor to avoid instantiation.
        /// </summary>
        private AppState() {}

        /// <summary>
        /// Returns the AppState instance.
        /// </summary>
        public static AppState Instance
        {
            get { return sInstance; }
        }

        #region Properties

        public League CurrentLeague
        {
            get { return new League(new LeagueBinaryFile(mCurrentFilename)); }
        }

        public string CurrentFilename
        {
            get { return mCurrentFilename; }
            set { mCurrentFilename = value; }
        }

        public string CurrentLeagueName
        {
            get { return mCurrentLeagueName; }
            set { mCurrentLeagueName = value; }
        }

        public string CurrentSeason
        {
            get { return mCurrentSeason; }
            set { mCurrentSeason = value; }
        }

        public string[] LeagueNames
        {
            get { return mLeagueNames; }
            set { mLeagueNames = value; }
        }

        public LeagueViewMode Mode
        {
            get { return mMode; }
            set { mMode = value; }
        }

        public IList DownloadLeagueList
        {
            get { return mDownloadLeagueList; }
        }

        public string LiveUpdateLog
        {
            get { return mLiveUpdateLog; }
            set { mLiveUpdateLog = value; }
        }

        #endregion Properties

        /// <summary>
        /// Loads settings from a file.
        /// </summary>
        public void Load()
        {
            BinaryReader reader = null;

            try
            {
                reader = new BinaryReader(
                    File.OpenRead(Path.Combine(Utils.AppDirectory, cSettingsFilename)));
                int fileVersion = reader.ReadByte();

                if (fileVersion >= 1)
                {
                    UTF8Encoding encoding = new UTF8Encoding();

                    mCurrentLeagueName = (encoding.GetString(
                        reader.ReadBytes(cLeagueNameChars * 4), 0, cLeagueNameChars * 4)).TrimEnd('\0');

                    mCurrentSeason = (encoding.GetString(
                        reader.ReadBytes(cSeasonChars * 4), 0, cSeasonChars * 4)).TrimEnd('\0');

                    int leagueNameCount = (int)reader.ReadByte();
                    mLeagueNames = new string[leagueNameCount];
                    for (int i = 0; i < leagueNameCount; i++)
                    {
                        mLeagueNames[i] = (encoding.GetString(
                            reader.ReadBytes(cLeagueNameChars * 4), 0, cLeagueNameChars * 4)).TrimEnd('\0');
                    }
                }

                if (fileVersion >= 2)
                {
                    int downloadLeagueCount = (int)reader.ReadByte();
                    mDownloadLeagueList.Clear();
                    for (int i = 0; i < downloadLeagueCount; i++)
                    {
                        mDownloadLeagueList.Add((int)reader.ReadInt16());
                    }
                }
            }
            catch (IOException)
            {
                // Since the settings file isn't essential we can
                // safely ignore any IOExceptions.
            }
            finally
            {
                if (reader != null) reader.Close();
            }

        }

        /// <summary>
        /// Save settings to a file.
        /// </summary>
        public void Save()
        {
            BinaryWriter writer = new BinaryWriter(
                File.OpenWrite(Path.Combine(Utils.AppDirectory, cSettingsFilename)));

            try
            {
                // File version 1
                writer.Write(Convert.ToByte(cFileVersion));
                writer.Write(Utils.GetUtf8Bytes((mCurrentLeagueName==null) ? "" : mCurrentLeagueName, cLeagueNameChars));
                writer.Write(Utils.GetUtf8Bytes((mCurrentSeason==null) ? "" : mCurrentSeason, cSeasonChars));
                writer.Write(Convert.ToByte(mLeagueNames.Length));
                for (int i = 0; i < mLeagueNames.Length; i++)
                {
                    writer.Write(Utils.GetUtf8Bytes(mLeagueNames[i], cLeagueNameChars));
                }

                // File version 2
                writer.Write(Convert.ToByte(mDownloadLeagueList.Count));
                for (int i = 0; i < mDownloadLeagueList.Count; i++)
                {
                    writer.Write(Convert.ToInt16((int)mDownloadLeagueList[i]));
                }
            }
            finally
            {
                if (writer != null) writer.Close();
            }        
        }


        /// <summary>
        /// Handles logging events during the download.
        /// </summary>
        public void LiveUpdate_Log(object sender, string message)
        {
            this.LiveUpdateLog += message;
        }        

    }
}
