/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.2.0 May 2005
 * Copyright (c) 2004-05 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Drawing;
using System.Windows.Forms;

namespace MBovin.TeamStats.Controls
{
	/// <summary>
	/// Panel control with vertical scrolling.
	/// </summary>
	public class ScrollPanel : System.Windows.Forms.Control
	{
        private Panel      mInnerPanel;
        private Panel      mOuterPanel;
        private VScrollBar mVScrollBar;


        /// <summary>
        /// Constructor.
        /// </summary>
        public ScrollPanel() : base()
        {
            mOuterPanel          = new Panel();
            mOuterPanel.Location = new Point(1, 1);
            base.Controls.Add(mOuterPanel);

            mInnerPanel = new Panel();
            mOuterPanel.Controls.Add(mInnerPanel);

            mVScrollBar              = new VScrollBar();
            mVScrollBar.Top          = 0;
            mVScrollBar.Width        = 14;
            mVScrollBar.Minimum      = 0;
            mVScrollBar.SmallChange  = 20;
            mVScrollBar.ValueChanged += new EventHandler(VScrollBar_ValueChanged);
            base.Controls.Add(mVScrollBar);
        }

        /// <summary>
        /// Overloaded. Returns the Controls-collection of the inner panel.
        /// </summary>
        public new Control.ControlCollection Controls
        {
            get { return mInnerPanel.Controls; }
        }

        /// <summary>
        /// Returns the maximum Bottom value of the child controls
        /// of the specified container control.
        /// </summary>
        private int GetChildMaxBottom(Control container)
        {
            int maxBottom = 0;

            // Loop over the controls and find the maximum Bottom value
            foreach (Control control in container.Controls)
            {
                maxBottom = Math.Max(maxBottom, control.Bottom);
            }

            return maxBottom;            
        }

        /// <summary>
        /// Resizes the panels and the scrollbar.
        /// </summary>
        protected override void OnResize(EventArgs e)
        {
            base.OnResize(e);

            mOuterPanel.Size        = new Size(this.Width - mVScrollBar.Width - 2, this.Height - 2);
            mInnerPanel.Width       = mOuterPanel.Width;

            mVScrollBar.Left        = this.Width - mVScrollBar.Width;
            mVScrollBar.Height      = this.Height;
            mVScrollBar.LargeChange = mOuterPanel.Height;
        }

        /// <summary>
        /// Paints the control.
        /// </summary>
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            // Resize inner panel and scrollbar
            mInnerPanel.Height  = GetChildMaxBottom(mInnerPanel);
            mVScrollBar.Maximum = mInnerPanel.Height;

            // Draw a border
            e.Graphics.DrawRectangle(new Pen(Color.Black), 0, 0, this.Width-1, this.Height-1);
        }

        /// <summary>
        /// Event handler for when scroll bar value is changed.
        /// </summary>
        private void VScrollBar_ValueChanged(object sender, EventArgs e)
        {
            mInnerPanel.Top = -mVScrollBar.Value;
        }

    }
}
