/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.0.0 July 2004
 * Copyright (c) 2004 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;

namespace MBovin.TeamStats.Core
{
	/// <summary>
	/// Delegate for filtering matches.
	/// </summary>
    public delegate bool MatchFilter(Match match, params object[] args);
    

    /// <summary>
	/// Implements a League.
	/// </summary>
	public class League
	{
        private ILeagueStore mLeagueStore;

        private string mName;
        private int    mYear;
        private bool   mIsSplit;
        private int    mTeamCount;
        private int    mRoundCount;
        private int    mMatchCount;
        private int    mPlayedMatchCount;
        private int    mWinPoints;
        private int    mDrawPoints;
        private int    mLossPoints;
        private long   mTableLines;
        private TableSortMethod mTableSort;

        private ArrayList mTeamList;

        
        /// <summary>
        /// Constructor.
        /// </summary>
        public League() 
        {
            mRoundCount = 1;
        }

        /// <summary>
        /// Constructor. Creates a league by loading data 
        /// from the specified LeagueStore.
        /// </summary>
        public League(ILeagueStore leagueStore)
		{
            mLeagueStore = leagueStore;
            mLeagueStore.LoadHeader(this);
		}

        #region Properties

        public string Name
        {
            get { return mName; }
            set { mName = value; }
        }

        public int Year
        {
            get { return mYear; }
            set { mYear = value; }
        }

        public bool IsSplit
        {
            get { return mIsSplit; }
            set { mIsSplit = value; }
        }

        public string Season
        {
            get
            {
                return (mIsSplit) ? 
                    mYear.ToString() + "-" + ((mYear+1)%100).ToString("00") :
                    mYear.ToString();
            }
        }

        public string FullName
        {
            get { return this.Name + " " + this.Season; }
        }

        public int TeamCount
        {
            get { return mTeamCount; }
            set { mTeamCount = value; }
        }

        public int RoundCount
        {
            get { return mRoundCount; }
            set { mRoundCount = value; }
        }

        public int MatchCount
        {
            get { return mMatchCount; }
            set { mMatchCount = value; }
        }

        public int PlayedMatchCount
        {
            get { return mPlayedMatchCount; }
            set { mPlayedMatchCount = value; }
        }

        public int WinPoints
        {
            get { return mWinPoints; }
            set { mWinPoints = value; }
        }

        public int DrawPoints
        {
            get { return mDrawPoints; }
            set { mDrawPoints = value; }
        }

        public int LossPoints
        {
            get { return mLossPoints; }
            set { mLossPoints = value; }
        }
        
        public TableSortMethod TableSort
        {
            get { return mTableSort; }
            set { mTableSort = value; }
        }

        public long TableLines
        {
            get { return mTableLines; }
            set { mTableLines = value; }
        }

        public ILeagueStore LeagueStore
        {
            get { return mLeagueStore; }
            set { mLeagueStore = value; }
        }

        #endregion

        /// <summary>
        /// Returns the team with the specfied index.
        /// </summary>
        public Team GetTeam(int teamIndex)
        {
            if (mTeamList == null)
            {
                GetTeams();
            }
            return (Team)mTeamList[teamIndex];
        }

        /// <summary>
        /// Returns the teams of this league as Team objects.
        /// </summary>
        public ArrayList GetTeams()
        {
            if (mTeamList == null)
            {
                mTeamList = mLeagueStore.LoadTeams(this);
            }
            return mTeamList;
        }

        /// <summary>
        /// Returns an array with the positions of the table lines.
        /// </summary>
        public int[] GetTableLines()
        {
            ArrayList linesList = new ArrayList();

            for (int i = 0; i < mTeamCount; i++)
            {
                if (IsTableLine(i + 1))
                {
                    linesList.Add(i+1);
                }
            }

            return (int[])linesList.ToArray(new Int32().GetType());
        }

        /// <summary>
        /// Returns true if there is a line in the table after
        /// the specified position.
        /// </summary>
        public bool IsTableLine(int position)
        {
            return ((long)Math.Pow(2, (position - 1)) & mTableLines) > 0;
        }

        /// <summary>
        /// Returns the latest round which has a match played.
        /// </summary>
        public int GetLastPlayedRound()
        {
            int       lastPlayedRound = 1;
            Match     match;
            ArrayList matchList = mLeagueStore.LoadMatches(this);

            for (int i = matchList.Count-1; i >= 0; i--)
            {
                match = (Match)matchList[i];
                if (match.IsPlayed)
                {
                    lastPlayedRound = match.Round;
                    break;
                }
            }

            return lastPlayedRound;
        }

        /// <summary>
        /// Returns a list of matches.
        /// </summary>
        /// <param name="onlyPlayed">Include only played games.</param>
        public ArrayList GetMatches(bool onlyPlayed)
        {
            return mLeagueStore.LoadMatches(
                this, 
                new MatchFilter(this.MatchFilterPlayed), 
                onlyPlayed);
        }

        /// <summary>
        /// Filter for returning only played matches.
        /// </summary>
        public bool MatchFilterPlayed(Match match, params object[] args)
        {
            bool onlyPlayed = (bool)args[0];
            return (!onlyPlayed || match.IsPlayed);
        }

        /// <summary>
        /// Returns the matches of the specified round.
        /// </summary>
        public ArrayList GetMatchesForRound(int round)
        {
            return mLeagueStore.LoadMatches(this, new MatchFilter(this.MatchFilterRound), round);
        }

        /// <summary>
        /// Filter for returning matches for a certain round.
        /// </summary>
        public bool MatchFilterRound(Match match, params object[] args)
        {
            return (match.Round == (int)args[0]);
        }

        /// <summary>
        /// Returns the matches for the spcified team.
        /// </summary>
        public ArrayList GetMatchesForTeam(int teamIndex, bool includeHome, bool includeAway)
        {
            return mLeagueStore.LoadMatches(
                this, new MatchFilter(this.MatchFilterTeam), teamIndex, includeHome, includeAway);
        }

        /// <summary>
        /// Filter for returning matches for a certain team.
        /// </summary>
        public bool MatchFilterTeam(Match match, params object[] args)
        {
            int  teamIndex = (int)args[0];
            bool includeHome = (bool)args[1];
            bool includeAway = (bool)args[2];

            return ((match.HomeTeamId == teamIndex) && includeHome) || 
                   ((match.AwayTeamId == teamIndex) && includeAway);
        }

        /// <summary>
        /// Creates the specified number of teams. The teams are 
        /// added to mTableTeamList and saved.
        /// </summary>
        public void AddTeams(int teamCount)
        {
            mTeamCount = teamCount;
            ArrayList teamList = new ArrayList(mTeamCount);
            for (int i = 0; i < mTeamCount; i++)
            {
                teamList.Add(new Team(this, i, "Team" + (i + 1)));
            }

            mLeagueStore.SaveHeader(this);
            mLeagueStore.SaveTeams(this, teamList);
        }

        /// <summary>
        /// Adds a match to the league.
        /// </summary>
        public void AddMatch(Match match)
        {            
            ArrayList matchList = mLeagueStore.LoadMatches(this);
            
            matchList.Add(match);            
            mMatchCount++;
            if (match.IsPlayed)
            {
                mPlayedMatchCount++;
            }

            Save();
            
            matchList.Sort();
            mLeagueStore.SaveMatches(this, matchList);
        }

        /// <summary>
        /// Deletes a match from the league.
        /// </summary>
        public void DeleteMatch(Match match)
        {
            ArrayList matchList = mLeagueStore.LoadMatches(this);

            matchList.RemoveAt(match.Index);
            mMatchCount--;
            if (match.IsPlayed)
            {
                mPlayedMatchCount--;
            }

            Save();            
            mLeagueStore.SaveMatches(this, matchList);
        }

        /// <summary>
        /// Loads all the matches of the league, sorts them (on the matchdate)
        /// and then saves them again. 
        /// Use this method when match dates have changed.
        /// </summary>
        public void SortMatches()
        {
            // Load matches and sort them
            ArrayList matchList = mLeagueStore.LoadMatches(this);
            matchList.Sort();
            mLeagueStore.SaveMatches(this, matchList);
        }

        /// <summary>
        /// Increases the number of rounds for this league.
        /// </summary>
        public void AddRound()
        {
            mRoundCount++;
            Save();
        }

        /// <summary>
        /// Decreases the number of rounds for this league. An exception is 
        /// thrown if the last round contains any matches.
        /// </summary>
        public void DeleteLastRound()
        {
            ArrayList matches = GetMatchesForRound(mRoundCount);
            if (matches.Count > 0) 
            {
                throw new Exception("Last round is not empty");
            }

            mRoundCount--;
            Save();
        }

        /// <summary>
        /// Saves the file header.
        /// </summary>
        public void Save()
        {
            mLeagueStore.SaveHeader(this);
        }

    }

}
