/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.0.0 July 2004
 * Copyright (c) 2004 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;

namespace MBovin.TeamStats.Core
{
	/// <summary>
	/// This class calculates league statistics.
	/// </summary>
	public class LeagueStats
	{
        private League mLeague;
        
        private int mHomeWins;
        private int mDraws;
        private int mAwayWins;
        private int mHomeGoals;
        private int mAwayGoals;

        private ArrayList mFrequentResultList = new ArrayList(10);

		
        /// <summary>
        /// Constructor.
        /// </summary>
        public LeagueStats(League league)
		{
            mLeague = league;
            Build();
		}

        #region Properties

        public int Played
        {
            get { return mHomeWins + mDraws + mAwayWins; }
        }

        public int HomeWins
        {
            get { return mHomeWins; }
        }

        public float HomeWinPercent
        {
            get { return (this.Played > 0) ? ((float)mHomeWins / (float)this.Played) : 0; }
        }

        public int Draws
        {
            get { return mDraws; }
        }

        public float DrawPercent
        {
            get { return (this.Played > 0) ? ((float)mDraws / (float)this.Played) : 0; }
        }

        public int AwayWins
        {
            get { return mAwayWins; }
        }

        public float AwayWinPercent
        {
            get { return (this.Played > 0) ? ((float)mAwayWins / (float)this.Played) : 0; }
        }

        public int Goals
        {
            get { return mHomeGoals + mAwayGoals; }
        }

        public float GoalAvg
        {
            get { return (this.Played > 0) ? ((float)this.Goals / (float)this.Played) : 0; }
        }

        public int HomeGoals
        {
            get { return mHomeGoals; }
        }

        public float HomeGoalAvg
        {
            get { return (this.Played > 0) ? ((float)mHomeGoals / (float)this.Played) : 0; }
        }

        public int AwayGoals
        {
            get { return mAwayGoals; }
        }

        public float AwayGoalAvg
        {
            get { return (this.Played > 0) ? ((float)mAwayGoals / (float)this.Played) : 0; }
        }

        #endregion

        /// <summary>
        /// Returns the most frequent result at the specified index.
        /// </summary>
        public Result GetFrequentResult(int index)
        {
            return (index > -1 && index < mFrequentResultList.Count) ?
                mFrequentResultList[index] as Result :
                null;
        }

        /// <summary>
        /// Builds the statistics.
        /// </summary>
        private void Build()
        {
            ArrayList matchList = mLeague.GetMatches(true);

            foreach (Match match in matchList)
            {
                if      (match.HomeGoals > match.AwayGoals) mHomeWins++;
                else if (match.HomeGoals < match.AwayGoals) mAwayWins++;
                else mDraws++;

                mHomeGoals += match.HomeGoals;
                mAwayGoals += match.AwayGoals;

                AddResult(match.HomeGoals, match.AwayGoals);
            }

            mFrequentResultList.Sort();
        }

        /// <summary>
        /// Adds a result to the the most frequent result list.
        /// </summary>
        private void AddResult(int homeGoals, int awayGoals)
        {
            bool resultFound = false;
            
            // Loop through all the results in the list
            foreach (Result result in mFrequentResultList)
            {
                // If the result is found -> bump the count for
                // that result and exit loop
                if (result.IsResult(homeGoals, awayGoals))
                {
                    result.Count++;
                    resultFound = true;
                    break;
                }
            }

            // This was a new result -> add it to the list
            if (!resultFound)
            {
                mFrequentResultList.Add(new Result(homeGoals, awayGoals));
            }
        }

	}


    /// <summary>
    /// This class is used for handling most frequent results.
    /// </summary>
    public class Result : IComparable
    {
        private int mHomeGoals;
        private int mAwayGoals;
        private int mCount;


        /// <summary>
        /// Constructor.
        /// </summary>
        public Result(int homeGoals, int awayGoals)
        {
            mHomeGoals = homeGoals;
            mAwayGoals = awayGoals;
            mCount = 1;
        }

        #region Properties

        public int HomeGoals
        {
            get { return mHomeGoals; }
        }

        public int AwayGoals
        {
            get { return mAwayGoals; }
        }

        public int Count
        {
            get { return mCount; }
            set { mCount = value; }
        }

        #endregion

        /// <summary>
        /// Returns true if this instance represents the specified result.
        /// </summary>
        public bool IsResult(int homeGoals, int awayGoals)
        {
            return (homeGoals == mHomeGoals) && (awayGoals == mAwayGoals);
        }

        /// <summary>
        /// Compares the count of this result to another result.
        /// </summary>
        public int CompareTo(object obj)
        {
            Result other = obj as Result;

            if      (this.Count > other.Count) return -1;
            else if (this.Count < other.Count) return 1;
            else                               return 0;
        }
        
    }

}
