/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.0.0 July 2004
 * Copyright (c) 2004 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;

namespace MBovin.TeamStats.Core
{

    /// <summary>
    /// Enumeration for different methods of sorting the table.
    /// </summary>
    public enum TableSortMethod
    {
        GoalDiff_GoalsFor,
        GoalDiff_Wins,
        Wins_GoalDiff,
        Wins_GoalsFor,
        GoalsFor_GoalDiff,
        GoalsFor_Wins
    }
    

    /// <summary>
	/// Implements a table.
	/// </summary>
    public class Table
    {
        private League    mLeague;
        private ArrayList mTeamList;

        private bool mIncludeHome;
        private bool mIncludeAway;
        private bool mIsForward;
        private int  mMaxMatches;


        /// <summary>
        /// Constructor.
        /// </summary>
        public Table(League league, 
                     bool includeHome, 
                     bool includeAway, 
                     bool isForward, 
                     int maxMatches)
        {
            mLeague      = league;
            mIncludeHome = includeHome;
            mIncludeAway = includeAway;
            mIsForward   = isForward;
            mMaxMatches  = maxMatches;

            Build();
            Sort();
        }

        #region Properties

        public ArrayList TeamList
        {
            get { return mTeamList; }
        }

        #endregion


        /// <summary>
        /// Returns the TableTeam for the specified team.
        /// </summary>
        public TableTeam GetTableTeam(Team team)
        {
            int pos = 1;

            // Loop through the table teams
            foreach (TableTeam tableTeam in mTeamList)
            {
                if (tableTeam.Index == team.Index)
                {
                    // Update the table position of this table team
                    tableTeam.Position = pos;
                    // Table team found -> bail out
                    return tableTeam;
                }
                pos++;
            }

            return null;
        }
        
        /// <summary>
        /// Builds the table by looping over all the matches.
        /// </summary>
        private void Build()
        {
            // Get the teams
            ArrayList teamList = mLeague.GetTeams();
            mTeamList = new ArrayList(teamList.Count);
            foreach (Team team in teamList)
            {
                mTeamList.Add(new TableTeam(team));
            }
            // Get the matches
            ArrayList matchList = mLeague.GetMatches(true);
            
            // Check in which direction we should loop
            int start, stop, step;
            if (mIsForward) 
            {
                start = 0;
                stop  = matchList.Count;
                step  = 1;
            }
            else
            {
                start = matchList.Count-1;
                stop  = -1;
                step  = -1;
            }

            // Loop through the matches
            for (int i = start; i != stop; i += step)
            {
                Match match = (Match)matchList[i];
                if (mIncludeHome)
                {
                    ((TableTeam)mTeamList[match.HomeTeamId]).AddMatch(match, mMaxMatches);
                }
                if (mIncludeAway)
                {
                    ((TableTeam)mTeamList[match.AwayTeamId]).AddMatch(match, mMaxMatches);
                }
            }

        }


        /// <summary>
        /// Sorts the table.
        /// </summary>
        private void Sort()
        {
            // Only include bonus points on "normal" table (over all matches)
            bool useBonusPoints = (mIncludeHome && mIncludeAway && (mMaxMatches == -1));

            // Calculate points and goaldif for each team
            foreach (TableTeam team in mTeamList)
            {
                team.SumUp(mLeague.WinPoints, 
                           mLeague.DrawPoints, 
                           mLeague.LossPoints, 
                           useBonusPoints);
            }

            // Sort the team array
            mTeamList.Sort();
        }

	}
}
