/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.0.0 July 2004
 * Copyright (c) 2004 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;

namespace MBovin.TeamStats.Core
{
	/// <summary>
	/// Implements a table team.
	/// </summary>
	public class TableTeam : IComparable 
	{
        private Team mTeam;
        private int  mPosition;
        private int  mPlayed;
        private int  mWon;
        private int  mDrawn;
        private int  mLost;
        private int  mFor;
        private int  mAgainst;
        private int  mGoalDiff;
        private int  mPoints;


        /// <summary>
        /// Constructor.
        /// </summary>
        public TableTeam(Team team)
        {
            mTeam = team;
        }

        #region Properties

        public string Name
        {
            get { return mTeam.Name; }
        }

        public int Index
        {
            get { return mTeam.Index; }
        }

        public int Position
        {
            get { return mPosition; }
            set { mPosition = value; }
        }

        public int Played
        {
            get { return mPlayed; }
        }

        public int Won
        {
            get { return mWon; }
        }

        public float WonPercent
        {
            get { return (mPlayed > 0) ? ((float)mWon / (float)mPlayed) : 0; }
        }

        public int Drawn
        {
            get { return mDrawn; }
        }

        public float DrawnPercent
        {
            get { return (mPlayed > 0) ? ((float)mDrawn / (float)mPlayed) : 0; }
        }

        public int Lost 
        {
            get { return mLost; }
        }

        public float LostPercent
        {
            get { return (mPlayed > 0) ? ((float)mLost / (float)mPlayed) : 0; }
        }

        public int For 
        {
            get { return mFor; }
        }

        public float ForAvg
        {
            get { return (mPlayed > 0) ? ((float)mFor / (float)mPlayed) : 0; }
        }

        public int Against
        {
            get { return mAgainst; }
        }

        public float AgainstAvg
        {
            get { return (mPlayed > 0) ? ((float)mAgainst / (float)mPlayed) : 0; }
        }

        public int GoalDiff
        {
            get { return mGoalDiff; }
        }

        public int Points
        {
            get { return mPoints; }
        }
        
        #endregion

        /// <summary>
        /// Adds a game for this table team.
        /// </summary>
        public void AddMatch(Match match, int maxMatches)
        {
            // Exit method if maxMatches is already reached
            if (maxMatches > -1 && mPlayed >= maxMatches) 
            {
                return;
            }

            int goalsFor;
            int goalsAgainst;
            if (match.HomeTeamId == mTeam.Index) 
            {
                goalsFor     = match.HomeGoals;
                goalsAgainst = match.AwayGoals;
            }
            else 
            {
                goalsFor     = match.AwayGoals;
                goalsAgainst = match.HomeGoals;
            }

            // Add result (won, draw, lost)
            mPlayed++;

            if (goalsFor > goalsAgainst) 
            {
                mWon++;
            }
            else if (goalsAgainst > goalsFor) 
            {
                mLost++;
            }
            else 
            {
                mDrawn++;
            }

            // Add goals
            mFor     += goalsFor;
            mAgainst += goalsAgainst;
        }

        /// <summary>
        /// Sums up the points and goal difference for a team.
        /// </summary>
        public void SumUp(int winPoints, int drawPoints, int lossPoints, bool includeBonusPoints)
        {
            mPoints = (mWon   * winPoints) + 
                      (mDrawn * drawPoints) + 
                      (mLost  * lossPoints);

            if (includeBonusPoints)
            {
                mPoints += mTeam.BonusPoints;
            }

            mGoalDiff = mFor - mAgainst;
        }

        #region IComparable Members

        /// <summary>
        /// Compares this team to another. Used by Array.Sort().
        /// </summary>
        public int CompareTo(object obj)
        {
            TableTeam enemy = (TableTeam)obj;
            int n = 0;
            int returnValue;

            while ((returnValue = CompareValue(enemy, n++)) == 0)
                ;
            
            return returnValue;
        }

        #endregion

        /// <summary>
        /// Compares the n:th value of this team with another team's value.
        /// </summary>
        /// <returns>1 if the other team is better, 0 if equal and -1 
        /// if this team is better.</returns>
        private int CompareValue(TableTeam enemy, int n)
        {
            int teamValue  = 0;
            int enemyValue = 0;

            switch(n) 
            {
                case 0:
                    teamValue  = mPoints;
                    enemyValue = enemy.mPoints;
                    break;
                case 1:
                    teamValue  = mTeam.HiddenPoints; 
                    enemyValue = enemy.mTeam.HiddenPoints; 
                    break;
                case 2:
                    teamValue  = GetCompareValue(2);
                    enemyValue = enemy.GetCompareValue(2);
                    break;
                case 3:
                    teamValue  = GetCompareValue(3);
                    enemyValue = enemy.GetCompareValue(3);
                    break;
                case 4:
                    teamValue  = GetCompareValue(4);
                    enemyValue = enemy.GetCompareValue(4);
                    break;
                case 5:
                    enemyValue = this.Name.CompareTo(enemy.Name);
                    break;
                case 6:
                    teamValue  = mTeam.Index;
                    enemyValue = enemy.mTeam.Index;
                    break;

            }
        
            if      (teamValue < enemyValue) return 1;
            else if (teamValue > enemyValue) return -1;
            else                             return 0;                
        }

        /// <summary>
        /// Returns the n:th value to use when comparing two teams. 
        /// </summary>
        private int GetCompareValue(int n) 
        {
            int returnValue = 0;

            switch (mTeam.League.TableSort) 
            {
                case TableSortMethod.GoalDiff_GoalsFor :
                    switch (n) 
                    {
                        case 2 : returnValue = mGoalDiff; break;
                        case 3 : returnValue = mFor; break;
                        case 4 : returnValue = mWon; break;
                    }
                    break;
    
                case TableSortMethod.GoalDiff_Wins :
                    switch (n) 
                    {
                        case 2 : returnValue = mGoalDiff; break;
                        case 3 : returnValue = mWon; break;
                        case 4 : returnValue = mFor; break;
                    }
                    break;

                case TableSortMethod.Wins_GoalDiff :
                    switch (n) 
                    {
                        case 2 : returnValue = mWon; break;
                        case 3 : returnValue = mGoalDiff; break;
                        case 4 : returnValue = mFor; break;
                    }
                    break;

                case TableSortMethod.Wins_GoalsFor :
                    switch (n) 
                    {
                        case 2 : returnValue = mWon; break;
                        case 3 : returnValue = mFor; break;
                        case 4 : returnValue = mGoalDiff; break;
                    }
                    break;
    
                case TableSortMethod.GoalsFor_GoalDiff :
                    switch (n) 
                    {
                        case 2 : returnValue = mFor; break;
                        case 3 : returnValue = mGoalDiff; break;
                        case 4 : returnValue = mWon; break;
                    }
                    break;

                case TableSortMethod.GoalsFor_Wins :
                    switch (n) 
                    {
                        case 2 : returnValue = mFor; break;
                        case 3 : returnValue = mWon; break;
                        case 4 : returnValue = mGoalDiff; break;
                    }
                    break;
            }
        
            return returnValue;
        }

    }
}
