/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;
using System.IO;
using System.Net;
using System.Threading;
using MBovin.TeamStats.Core;
using MBovin.TeamStats.Util;

namespace MBovin.TeamStats.LiveUpdate
{
    /// <summary>
    /// Data structure for information about a downloadable league.
    /// </summary>
    public struct DownloadableLeague
    {
        public int    Id;
        public string Name;
        public string Country;

        public override bool Equals(object obj)
        {
            return (this.Id == ((DownloadableLeague)obj).Id);
        }

        public override int GetHashCode()
        {
            return base.GetHashCode();
        }
    }

	/// <summary>
	/// Class for downloading league files from the internet.
	/// </summary>
	public class LeagueDownloader : LiveUpdateAsyncBase
	{
        private const string cLeagueBaseUrl = "http://www.aragon.ws/soccerdb/util/ppccreate.php";

        private IList  mLeagueList;
        private League mLeague;


		/// <summary>
		/// Constructor.
		/// </summary>
        public LeagueDownloader() {}

        /// <summary>
        /// Starts a thread that will download leagues asynchronously.
        /// </summary>
        /// <param name="leagueList">A list with league-id:s</param>
        public void DownloadLeaguesAsync(IList leagueList)
        {
            if (!mIsThreadRunning)
            {
                mCancelPending = false;
                mLeagueList = leagueList;
                Thread thread = new Thread(new ThreadStart(this.DownloadLeagues));
                thread.Start();
            }
        }

        /// <summary>
        /// Starts a thread that will download a single league asynchronously.
        /// </summary>
        /// <param name="league">The league to download</param>
        public void DownloadLeagueAsync(League league)
        {
            if (!mIsThreadRunning)
            {
                mCancelPending = false;
                mLeague = league;
                Thread thread = new Thread(new ThreadStart(this.DownloadLeague));
                thread.Start();
            }
        }

        /// <summary>
        /// Downloads leagues from a list.
        /// </summary>
        private void DownloadLeagues()
        {
            mIsThreadRunning = true;

            LogLine(Language.GetString("liveupdate", "downloadingleagues"));
            bool isError = false;
            foreach (int leagueId in  mLeagueList)
            {
                // Bail out of loop if the cancel flag has been set
                if (mCancelPending)
                {
                    LogLine(Language.GetString("liveupdate", "downloadcancelled"));
                    break;
                }

                // Url to download
                string url = cLeagueBaseUrl + "?id=" + leagueId;

                try
                {
                    // Download the file
                    string filename = DownloadLeagueFile(url);
                    LogLine(filename + "..." + Language.GetString("liveupdate", "ok"));
                }
                catch (WebException e)
                {
                    isError = true;
                    LogLine(url + "..." + Language.GetString("liveupdate", "failed") + " - " + e.Status.ToString());
                }
                catch (Exception e)
                {
                    isError = true;
                    LogLine(url + "..." + Language.GetString("liveupdate", "failed") + " - " + e.Message);
                }

                FireLiveUpdateStateChanged(LiveUpdateEvent.LeagueDownloaded);
            }
            if (isError)
            {
                LogLine("\r\n" + Language.GetString("liveupdate", "errors"));
            }

            FireLiveUpdateStateChanged(LiveUpdateEvent.Finished);

            mIsThreadRunning = false;
        }

        /// <summary>
        /// Downloads a single league.
        /// </summary>
        private void DownloadLeague()
        {
            mIsThreadRunning = true;

            LogLine(Language.GetString("liveupdate", "downloadingleague"));

            // Url to download
            string url = string.Format("{0}?name={1}&year={2}", cLeagueBaseUrl, mLeague.Name, mLeague.Year);
            
            try
            {
                // Download the file
                string filename = DownloadLeagueFile(url);
                LogLine(filename + "..." + Language.GetString("liveupdate", "ok"));
            }
            catch (WebException e)
            {
                LogLine(url + "..." + Language.GetString("liveupdate", "failed") + " - " + e.Status.ToString());
            }
            catch (LeagueNotFoundException e)
            {
                LogLine(url + "..." + Language.GetString("liveupdate", "failed") + " - " + e.Message);
                FireLiveUpdateStateChanged(LiveUpdateEvent.LeagueNotFound);
            }
            catch (Exception e)
            {
                LogLine(url + "..." + Language.GetString("liveupdate", "failed") + " - " + e.Message);
            }
            FireLiveUpdateStateChanged(LiveUpdateEvent.LeagueDownloaded);
            FireLiveUpdateStateChanged(LiveUpdateEvent.Finished);

            mIsThreadRunning = false;
        }

        /// <summary>
        /// Downloads the specified url and stores it in the program directory.
        /// Returns the name for the downloaded file.
        /// </summary>
        private string DownloadLeagueFile(string url)
        {
            HttpWebRequest  request  = null;
            HttpWebResponse response = null;
            BinaryReader    reader   = null;
            BinaryWriter    writer   = null;
            string          filename = null;
                        
            try
            {
                // Add some params for server-side logging to the query string
                url += GetQueryString();
                
                // Create the request object
                request = (HttpWebRequest)WebRequest.Create(url);
                request.Timeout = 30000;

                // Get the response object and open a reader
                response = (HttpWebResponse)request.GetResponse();
                reader   = new BinaryReader(response.GetResponseStream());

                // Get the filename and bail out if it's not valid
                filename = GetFilename(response);
                if (filename == null)
                {
                    throw new LeagueNotFoundException(Language.GetString("liveupdate", "invalidfile"));
                }

                // Create the full filename and open a writer to it
                string fullname = Path.Combine(Utils.AppDirectory, filename);
                writer = new BinaryWriter(
                    new FileStream(fullname, FileMode.Create, FileAccess.Write, FileShare.None));

                // Copy from the reader to the local file
                byte[] buf = new byte[256];
                int count  = reader.Read(buf, 0, 256);
                while (count > 0)
                {
                    writer.Write(buf, 0, count);
                    count = reader.Read(buf, 0, 256);
                }
            }
            finally
            {
                if (writer   != null) writer.Close();
                if (response != null) response.Close();
            }

            return filename;
        }

        /// <summary>
        /// Returns the filename for the specified HttpWebResponse. If the
        /// content type is not correct, null is returned.
        /// </summary>
        private string GetFilename(HttpWebResponse response)
        {
            string filename = null;

            // If content type is "application/x-teamstats" the filename
            // can be found in the content disposition
            if (response.ContentType.StartsWith("application/x-teamstats"))
            {
                // The content-disposition is assumed to look like this:
                // Content-Disposition: attachment; filename="filename.ts"
                string[] arrContentDisp = response.Headers["Content-Disposition"].Split(';');
                string[] arrFilename    = arrContentDisp[1].Split('=');
                filename = arrFilename[1].Trim(" \"".ToCharArray());
            }

            return filename;
        }

        /// <summary>
        /// Returns the query string to use with web requests. 
        /// The query string params are used for server-side logging.
        /// </summary>
        private string GetQueryString()
        {
            Version clrVersion = System.Environment.Version;
            Version osVersion  = System.Environment.OSVersion.Version;

            return string.Format("&tsVersion={0}&deviceId={1}&osVersion={2}.{3}.{4}&clrVersion={5}.{6}.{7}.{8}",
                MainForm.cLiveUpdateVersion,
                Utils.GetDeviceID(),
                osVersion.Major, osVersion.Minor, osVersion.Build,
                clrVersion.Major, clrVersion.Minor, clrVersion.Build, clrVersion.Revision);
        }
	}
}
