/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;
using System.IO;
using System.Net;
using System.Threading;
using System.Windows.Forms;
using System.Xml;
using MBovin.TeamStats.Util;

namespace MBovin.TeamStats.LiveUpdate
{
    /// <summary>
    /// Class that handles lists with leagues that are available for download.
    /// </summary>
    public class LeagueListManager : LiveUpdateAsyncBase
    {
        private const string cActiveListUrl = "http://www.aragon.ws/soccerdb/export/liveupdateleagues.php";
        private const string cArchiveListUrl = "http://www.aragon.ws/soccerdb/export/completegrouplist.php";
        private const string cActiveListFilename  = "activeleagues.xml";
        private const string cArchiveListFilename = "archiveleagues.xml";
        private const string cLastSyncActiveListFilename = "lastsyncactiveleagues.xml";

        private IList mAddedLeagueList;


        /// <summary>
        /// Constructor.
        /// </summary>
        public LeagueListManager() {}

        /// <summary>
        /// A list with leagues that have been added to the 
        /// server since the last LiveUpdate.
        /// </summary>
        public IList AddedLeagueList
        {
            get { return mAddedLeagueList; }
        }

        /// <summary>
        /// Full path to local xml with active leagues.
        /// </summary>
        private string ActiveFullpath
        {
            get { return Path.Combine(Utils.AppDirectory, cActiveListFilename); }
        }

        /// <summary>
        /// Full path to local xml with archive leagues.
        /// </summary>
        private string ArchiveFullpath
        {
            get { return Path.Combine(Utils.AppDirectory, cArchiveListFilename); }
        }

        /// <summary>
        /// Starts a thread that willl check the remote server to see 
        /// if any new leagues have been added since the last time 
        /// LiveUpdate was run. New leagues will be available in the 
        /// AddedLeagueList property.
        /// </summary>
        public void CheckForNewLeaguesAsync()
        {
            if (!mIsThreadRunning)
            {
                Thread thread = new Thread(new ThreadStart(this.CheckForNewLeagues));
                thread.Start();
            }
        }

        /// <summary>
        /// Checks the remote server to see if any new leagues have 
        /// been added since the last time LiveUpdate was run.
        /// New leagues will be available in the AddedLeagueList property.
        /// </summary>
        private void CheckForNewLeagues()
        {
            mIsThreadRunning = true;

            LogMsg(Language.GetString("liveupdate", "checkingnew"));

            string filename = Path.Combine(Utils.AppDirectory, cLastSyncActiveListFilename);
            
            // Locally stored list from last LiveUpatde
            IList localLeagueList  = ParseActiveXml(GetLocalXml(filename));
            
            // The current list from the server
            IList removeLeagueList = ParseActiveXml(GetRemoteXml(cActiveListUrl, filename));
            
            // Create a new list with added leagues
            mAddedLeagueList = CreateAddedLeagueList(localLeagueList, removeLeagueList);
            
            LogLine(Language.GetString("liveupdate", "done"));

            if (mAddedLeagueList.Count > 0)
            {
                LogLine(string.Format(
                    Language.GetString("liveupdate", "newleaguesfound"), mAddedLeagueList.Count) + "\r\n");
            }
            else
            {
                LogLine(Language.GetString("liveupdate", "nonewleaguesfound") + "\r\n");
            }

            FireLiveUpdateStateChanged(LiveUpdateEvent.CheckForNewLeaguesFinished);

            mIsThreadRunning = false;
        }

        /// <summary>
        /// Returns true if a local file with active leagues is 
        /// available and not too old.
        /// </summary>
        public bool IsLocalActiveListAvailable()
        {
            return File.Exists(this.ActiveFullpath) &&
                (File.GetCreationTime(this.ActiveFullpath).AddHours(8) > DateTime.Now);
        }

        /// <summary>
        /// Returns a list with active leagues.
        /// </summary>
        /// <returns>A list with DownloadableLeague objects</returns>
        public IList GetActiveList()
        {
            XmlDocument xmlDoc = IsLocalActiveListAvailable() ?
                GetLocalXml(this.ActiveFullpath) : 
                GetRemoteXml(cActiveListUrl, this.ActiveFullpath);

            return ParseActiveXml(xmlDoc);
        }

        /// <summary>
        /// Returns true if local file with archived leagues is available.
        /// </summary>
        public bool IsLocalArchiveListAvailable()
        {
            return File.Exists(this.ArchiveFullpath) &&
                (File.GetCreationTime(this.ArchiveFullpath).AddHours(8) > DateTime.Now);
        }

        /// <summary>
        /// Returns a list with leagues from the archive.
        /// </summary>
        /// <returns>A list with System.Windows.Forms.TreeNode objects.</returns>
        public IList GetArchiveList()
        {
            XmlDocument xmlDoc = IsLocalArchiveListAvailable() ?
                GetLocalXml(this.ArchiveFullpath) : 
                GetRemoteXml(cArchiveListUrl, this.ArchiveFullpath);

            return ParseArchiveXml(xmlDoc);
        }

        /// <summary>
        /// Deletes local copies of league lists.
        /// </summary>
        public void DeleteLocalFiles()
        {
            File.Delete(this.ActiveFullpath);
            File.Delete(this.ArchiveFullpath);
        }

        /// <summary>
        /// Returns xml from a local file.
        /// </summary>
        /// <param name="filename">Full path to xml-file</param>
        /// <returns>An XmlDocument representing the file</returns>
        private XmlDocument GetLocalXml(string filename)
        {
            XmlDocument xmlDoc = new XmlDocument();
            if (File.Exists(filename))
            {
                using (FileStream stream = new FileStream(filename, FileMode.Open))
                {
                    xmlDoc.Load(stream);
                }
            }
            return xmlDoc;
        }

        /// <summary>
        /// Returns xml from the remote server.
        /// </summary>
        /// <param name="url">Url to xml-file</param>
        /// <param name="filename">Full path to where a local copy will be stored. Use null
        /// if you don't want to save a local copy.</param>
        /// <returns>An XmlDocument representing the url</returns>
        private XmlDocument GetRemoteXml(string url, string filename)
        {
            XmlDocument xmlDoc = Utils.DownloadXmlDocument(url);
            if (filename != null)
            {
                using (FileStream stream = new FileStream(filename, FileMode.Create))
                {
                    xmlDoc.Save(stream);
                }
            }
            return xmlDoc;
        }

        /// <summary>
        /// Parses an xml-document with active leagues.
        /// </summary>
        /// <param name="xmlDoc">XmlDocument with league info</param>
        /// <returns>A list with DownloadableLeague-objects</returns>
        private IList ParseActiveXml(XmlDocument xmlDoc)
        {
            IList leagueList = new ArrayList();

            // Loop over each node in the xml-data and create a DownloadableLeague object
            foreach (XmlNode leagueNode in xmlDoc.GetElementsByTagName("league"))
            {
                DownloadableLeague league = new DownloadableLeague();
                league.Id       = int.Parse(leagueNode.Attributes["id"].Value);
                league.Name     = leagueNode["name"].InnerText;
                league.Country  = leagueNode["country"].InnerText;

                // Add league to list
                leagueList.Add(league);
            }

            return leagueList;
        }

        /// <summary>
        /// Parses an xml-document with leagues from the archive.
        /// </summary>
        /// <param name="xmlDoc">XmlDocument with league info</param>
        /// <returns>A list with TreeNode-objects</returns>
        private IList ParseArchiveXml(XmlDocument xmlDoc)
        {
            IList nodeList = new ArrayList();

            ProcessArchiveNode(xmlDoc.DocumentElement, nodeList, null);

            return nodeList;
        }

        /// <summary>
        /// Processes one node in the archive xml.
        /// </summary>
        /// <param name="node">Xml-node to process</param>
        /// <param name="nodeList">List to store top level TreeNodes in</param>
        /// <param name="parentTreeNode">The parent TreeNode, if any</param>
        private void ProcessArchiveNode(XmlNode node, IList nodeList, TreeNode parentTreeNode)
        {
            // Loop over all the children
            foreach (XmlNode child in node.ChildNodes)
            {
                if (child.NodeType == XmlNodeType.Element)
                {
                    TreeNode newTreeNode = new TreeNode(child.Attributes["name"].Value);
                    
                    // If this is a leaf node we should store the id (which is a SoccerDB groupId)
                    if (!child.HasChildNodes)
                    {
                        newTreeNode.Tag = Utils.StringToInt32(child.Attributes["id"].Value, 0);
                    }

                    // If no parent node this is a top level node (country) -> add to node list
                    if (parentTreeNode == null)
                    {
                        nodeList.Add(newTreeNode);
                    }
                    else // Add to parent node
                    {
                        parentTreeNode.Nodes.Add(newTreeNode);
                    }

                    // Process the child recursively
                    ProcessArchiveNode(child, nodeList, newTreeNode);
                }
            }
        }

        /// <summary>
        /// Creates a list with the leagues in newList that are not present in oldList.
        /// </summary>
        /// <param name="oldList">List with leagues from the last LiveUpdate</param>
        /// <param name="newList">List leagues currently available</param>
        /// <returns>List with newly added league</returns>
        private IList CreateAddedLeagueList(IList oldList, IList newList)
        {
            IList newLeagueList = new ArrayList();
            if ((oldList != null) && (oldList.Count > 0) && (newList != null))
            {
                foreach (DownloadableLeague newLeague in newList)
                {
                    if (!oldList.Contains(newLeague))
                    {
                        newLeagueList.Add(newLeague);
                    }
                }
            }
            return newLeagueList;
        }
    }
}
