/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;

namespace MBovin.TeamStats.LiveUpdate
{
    #region Support types
    /// <summary>
    /// Delegate for log events 
    /// </summary>
    public delegate void LogEventHandler(object sender, string message);

    /// <summary>
    /// Delegate for LiveUpdate events 
    /// </summary>
    public delegate void LiveUpdateEventHandler(object sender, LiveUpdateEvent liveUpdateEvent);

    /// <summary>
    /// LiveUpdate state change event.
    /// </summary>
    public enum LiveUpdateEvent
    {
        NoConnection,
        ConnectionOk,
        CheckForNewLeaguesFinished,
        LeagueDownloaded,
        LeagueNotFound,
        Finished
    }
    #endregion

    /// <summary>
    /// Abstract base class for LiveUpdate-classes that 
    /// perform asynchronous activities.
    /// </summary>
    public abstract class LiveUpdateAsyncBase
    {
        // Event for logging
        public event LogEventHandler Log;
        // Event for LiveUpdate state changes
        public event LiveUpdateEventHandler LiveUpdateStateChanged;

        // Flag for thread state
        protected bool mIsThreadRunning = false;
        // Cancel flag
        protected bool mCancelPending;


        /// <summary>
        /// Constructor
        /// </summary>
        public LiveUpdateAsyncBase() {}

        /// <summary>
        /// Tells the object to cancel any asynchronous 
        /// activities at first opportunity.
        /// </summary>
        public void CancelAsyncDownload()
        {
            mCancelPending = true;
        }

        /// <summary>
        /// Fires the LiveUpdateStateChanged event.
        /// </summary>
        /// <param name="liveUpdateEvent">The event that ocurred</param>
        protected void FireLiveUpdateStateChanged(LiveUpdateEvent liveUpdateEvent)
        {
            LiveUpdateStateChanged(this, liveUpdateEvent);
        }

        /// <summary>
        /// Logs a string to the LogEvent-listeners.
        /// </summary>
        protected void LogMsg(string s)
        {            
            Log(this, s);
        }

        /// <summary>
        /// Logs a string and a newline to the LogEvent-listeners.
        /// </summary>
        protected void LogLine(string s)
        {
            LogMsg(s + "\r\n");
        }
    }
}
