/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;
using System.Collections.Specialized;
using System.Drawing;
using System.IO;
using System.Resources;
using System.Windows.Forms;
using Microsoft.WindowsCE.Forms;
using MBovin.TeamStats.Core;
using MBovin.TeamStats.LiveUpdate;
using MBovin.TeamStats.UI;
using MBovin.TeamStats.Util;

namespace MBovin.TeamStats
{
    #region Support types
    /// <summary>
    /// Enumeration for user interface modes.
    /// </summary>
    public enum UIMode
    {
        Normal,
        NoLeagues
    }

    /// <summary>
    /// Enumeration for the league view modes.
    /// </summary>
    public enum LeagueViewMode
    {
        Table,
        Matches,
        Teams,
        League
    }
    #endregion

    /// <summary>
	/// The main form of this application.
	/// </summary>
    public class MainForm : System.Windows.Forms.Form
    {
        // Version constants
        public static readonly string cVersion           = "TeamStats v1.3.1";
        public static readonly string cLiveUpdateVersion = "PPC1.3";

        // The font used in buttons
        public static readonly Font cButtonFont = new Font("Tahoma", 8F, FontStyle.Regular);

        // Controls
        private TabControl mTabControlMain;
        private TablePage  mTabPageTable;
        private MatchPage  mTabPageMatches;
        private TeamPage   mTabPageTeams;
        private LeaguePage mTabPageLeague;

        private ComboBox mComboBoxLeague;
        private ComboBox mComboBoxSeason;

        private MainMenu mMainMenu;
        private MenuItem mMenuItemFile;
        private MenuItem mMenuItemNewLeague;
        private MenuItem mMenuItemDeleteLeague;
        private MenuItem mMenuItemEdit;
        private MenuItem mMenuItemEditLeague;
        private MenuItem mMenuItemEditTeams;
        private MenuItem mMenuItemEditFixtures;
        private MenuItem mMenuItemSep1;
        private MenuItem mMenuItemLeagueOrder;
        private MenuItem mMenuItemSep2;
        private MenuItem mMenuItemExit;

        private MenuItem mMenuItemLiveUpdate;
        private MenuItem mMenuItemSelectLeagues;
        private MenuItem mMenuItemDownloadSelected;
        private MenuItem mMenuItemSep3;
        private MenuItem mMenuItemDownloadThisLeague;
        private MenuItem mMenuItemDownloadArchive;
        private MenuItem mMenuItemSep4;
        private MenuItem mMenuItemLog;
        
        private MenuItem mMenuItemAbout;

        private ToolBar mToolBar;

        private Label mLabelNoLeagues;

        // mLeagues contains an item for each league, keyed by league name.
        // Each item is another hashtable containing filenames, keyed by season.
        private ListDictionary mLeagues;
    
        // Holds the current app state
        private AppState mAppState;


        /// <summary>
        /// Constructor.
        /// </summary>
        public MainForm()
        {
            // Get the appstate instance and load saved settings
            mAppState = AppState.Instance;
            mAppState.Load();

            // Load languauge file
            Language.GetInstance().Load();

            // Required for Windows Form Designer support
            InitializeComponent();

            // The table page is displayed when the program starts
            // so we need to build the gui for that page
            mTabPageTable.BuildGui();

            // Look for and load leagues
            InitLeagues(mAppState.CurrentLeagueName, mAppState.CurrentSeason);
        }

        #region Windows Form Designer generated code
        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            int screenHeight = Screen.PrimaryScreen.WorkingArea.Height;

            // menuItemNewLeague
            mMenuItemNewLeague        = new MenuItem();
            mMenuItemNewLeague.Text   = Language.GetString("mainform", "menuleaguenew");
            mMenuItemNewLeague.Click += new System.EventHandler(this.MenuItemNewLeague_Click);
            // menuItemDeleteLeague
            mMenuItemDeleteLeague        = new MenuItem();
            mMenuItemDeleteLeague.Text   = Language.GetString("mainform", "menuleaguedelete");
            mMenuItemDeleteLeague.Click += new System.EventHandler(this.MenuItemDeleteLeague_Click);
            // menuItemEditFixtures
            mMenuItemEditFixtures        = new MenuItem();
            mMenuItemEditFixtures.Text   = Language.GetString("mainform", "menuleagueeditfixtures");
            mMenuItemEditFixtures.Click += new System.EventHandler(this.MenuItemEditFixtures_Click);
            // menuItemEditTeams
            mMenuItemEditTeams        = new MenuItem();
            mMenuItemEditTeams.Text   = Language.GetString("mainform", "menuleagueeditteams");
            mMenuItemEditTeams.Click += new System.EventHandler(this.MenuItemEditTeams_Click);
            // menuItemEditLeague
            mMenuItemEditLeague        = new MenuItem();
            mMenuItemEditLeague.Text   = Language.GetString("mainform", "menuleagueeditleague");
            mMenuItemEditLeague.Click += new System.EventHandler(this.MenuItemEditLeague_Click);
            // menuItemEdit
            mMenuItemEdit      = new MenuItem();
            mMenuItemEdit.Text = Language.GetString("mainform", "menuleagueedit");
            mMenuItemEdit.MenuItems.Add(mMenuItemEditLeague);
            mMenuItemEdit.MenuItems.Add(mMenuItemEditTeams);
            mMenuItemEdit.MenuItems.Add(mMenuItemEditFixtures);
            // menuItemLeagueOrder
            mMenuItemLeagueOrder        = new MenuItem();
            mMenuItemLeagueOrder.Text   = Language.GetString("mainform", "menuleagueorder");
            mMenuItemLeagueOrder.Click += new System.EventHandler(this.MenuItemLeagueOrder_Click);
            // menuItemExit
            mMenuItemExit        = new MenuItem();
            mMenuItemExit.Text   = Language.GetString("mainform", "menuleagueexit");
            mMenuItemExit.Click += new System.EventHandler(this.MenuItemExit_Click);
            // 
            // menuItemFile
            // 
            mMenuItemFile = new MenuItem();
            mMenuItemFile.Text = Language.GetString("mainform", "menuleague");

            // mMenuItemSelectLeagues
            mMenuItemSelectLeagues      = new MenuItem();
            mMenuItemSelectLeagues.Text = Language.GetString("mainform", "menuliveupdateselect"); ;
            mMenuItemSelectLeagues.Click += new System.EventHandler(this.MenuItemSelectLeagues_Click);
            // mMenuItemDownloadSelected
            mMenuItemDownloadSelected        = new MenuItem();
            mMenuItemDownloadSelected.Text   = Language.GetString("mainform", "menuliveupdateupdateselected");
            mMenuItemDownloadSelected.Click += new System.EventHandler(this.MenuItemDownloadSelected_Click);
            // sep
            mMenuItemSep3 = new MenuItem();
            mMenuItemSep3.Text = "-";
            // mMenuItemDownloadThisLeague
            mMenuItemDownloadThisLeague        = new MenuItem();
            mMenuItemDownloadThisLeague.Text   = Language.GetString("mainform", "menuliveupdateupdatethis");
            mMenuItemDownloadThisLeague.Click += new System.EventHandler(this.MenuItemDownloadThisLeague_Click);
            // mMenuItemDownloadArchive
            mMenuItemDownloadArchive        = new MenuItem();
            mMenuItemDownloadArchive.Text   = Language.GetString("mainform", "menuliveupdatearchive");
            mMenuItemDownloadArchive.Click += new System.EventHandler(this.MenuItemDownloadArchive_Click);
            // sep
            mMenuItemSep4 = new MenuItem();
            mMenuItemSep4.Text = "-";
            // mMenuItemLog
            mMenuItemLog        = new MenuItem();
            mMenuItemLog.Text   = Language.GetString("mainform", "menuliveupdatelog");
            mMenuItemLog.Click += new System.EventHandler(this.MenuItemLog_Click);
            // 
            // menuItemLiveUpdate
            // 
            mMenuItemLiveUpdate      = new MenuItem();
            mMenuItemLiveUpdate.Text = Language.GetString("mainform", "menuliveupdate");
            
            // 
            // menuItemAbout
            // 
            mMenuItemAbout        = new MenuItem();
            mMenuItemAbout.Text   = Language.GetString("mainform", "menuabout");
            mMenuItemAbout.Click += new System.EventHandler(this.MenuItemAbout_Click);
            // 
            // menuItemSep
            // 
            mMenuItemSep1      = new MenuItem();
            mMenuItemSep1.Text = "-";
            mMenuItemSep2      = new MenuItem();
            mMenuItemSep2.Text = "-";

            // 
            // mainMenu
            // 
            mMainMenu = new MainMenu();
            mMainMenu.MenuItems.Add(mMenuItemFile);
            mMainMenu.MenuItems.Add(mMenuItemLiveUpdate);
            mMainMenu.MenuItems.Add(mMenuItemAbout);
            
            // 
            // tabControlMain
            // 
            this.mTabControlMain               = new TabControl();
            this.mTabControlMain.Parent        = this;
            this.mTabControlMain.Top           = 15;
            this.mTabControlMain.Size          = new System.Drawing.Size(244, screenHeight - 49);
            this.mTabControlMain.Font          = new System.Drawing.Font("Tahoma", 8F, System.Drawing.FontStyle.Regular);
            this.mTabControlMain.SelectedIndex = 0;
            this.mTabControlMain.SelectedIndexChanged += new System.EventHandler(this.TabControlMain_SelectedIndexChanged);
            this.mTabPageTable   = new TablePage(mAppState);
            this.mTabPageMatches = new MatchPage(mAppState);
            this.mTabPageTeams   = new TeamPage(mAppState);
            this.mTabPageLeague  = new LeaguePage(mAppState);
            this.mTabControlMain.Controls.Add(this.mTabPageTable);
            this.mTabControlMain.Controls.Add(this.mTabPageMatches);
            this.mTabControlMain.Controls.Add(this.mTabPageTeams);
            this.mTabControlMain.Controls.Add(this.mTabPageLeague);
            // 
            // comboBoxLeague
            // 
            this.mComboBoxLeague        = new ComboBox();
            this.mComboBoxLeague.Parent = this;
            this.mComboBoxLeague.Bounds = new Rectangle(2, screenHeight - 48, 128, 20);
            this.mComboBoxLeague.SelectedIndexChanged += new System.EventHandler(this.ComboBoxLeague_SelectedIndexChanged);
            // 
            // comboBoxSeason
            // 
            this.mComboBoxSeason        = new ComboBox();
            this.mComboBoxSeason.Parent = this;
            this.mComboBoxSeason.Bounds = new Rectangle(134, screenHeight - 48, 72, 20);
            this.mComboBoxSeason.SelectedIndexChanged += new System.EventHandler(this.ComboBoxSeason_SelectedIndexChanged);
            // 
            // toolBar
            // 
            mToolBar              = new ToolBar();
            mToolBar.Parent       = this;
            mToolBar.ButtonClick += new ToolBarButtonClickEventHandler(ToolBar_ButtonClick);
            // Toolbar icons are loaded in OnLoad()
            ToolBarButton toolBarButton = new ToolBarButton();
            toolBarButton.ImageIndex    = 0;
            toolBarButton.Style         = ToolBarButtonStyle.PushButton;
            mToolBar.Buttons.Add(toolBarButton);
            // 
            // MainForm
            // 
            ResourceManager resources = new ResourceManager(typeof(MainForm));
            this.ClientSize  = new System.Drawing.Size(244, screenHeight - 29);
            this.Text        = "TeamStats";
            this.MaximizeBox = false;
            this.Icon        = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Menu        = this.mMainMenu;
        }
        #endregion

        #region Events
        
        /// <summary>
        /// Peforms some tasks when the form (application) is loaded.
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            // Load images in the toolbar
            mToolBar.ImageList = new ImageList();
            mToolBar.ImageList.Images.Add(Utils.LoadEmbeddedIcon("MBovin.TeamStats.LiveUpdate.ico", 16, 16));            
        }

        /// <summary>
        /// This event is triggered when the application is about to finish.
        /// </summary>
        protected override void OnClosing(System.ComponentModel.CancelEventArgs e)
        {
            // Save the current settings before exiting
            mAppState.Save();
            
            // Delete locally cached files
            new LeagueListManager().DeleteLocalFiles();
            
            base.OnClosing(e);
        }

        /// <summary>
        /// Handle changes in the league combo box.
        /// </summary>
        private void ComboBoxLeague_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            FillSeasonCombo();
            mComboBoxSeason.SelectedIndex = 0;
        }
    
        /// <summary>
        /// Handle changes in the season combo box.
        /// </summary>
        private void ComboBoxSeason_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            ShowLeague();
        }

        /// <summary>
        /// Handle changes of active tab.
        /// </summary>
        private void TabControlMain_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            // If we're leaving the match page -> make sure to save any changed results
            if (mAppState.Mode == LeagueViewMode.Matches)
            {
                mTabPageMatches.SaveChangedResults();

                mTabPageTable.InvalidateData();
                mTabPageTeams.InvalidateData();
                mTabPageLeague.InvalidateData();
            }

            mAppState.Mode = (LeagueViewMode)mTabControlMain.SelectedIndex;

            // Calling BuildGui() will make sure the gui is built in case this
            // is the first time the page is displayed.
            PageBase tabPage = mTabControlMain.TabPages[mTabControlMain.SelectedIndex] as PageBase;
            tabPage.BuildGui();
        }

        /// <summary>
        /// Create new league.
        /// </summary>
        private void MenuItemNewLeague_Click(object sender, System.EventArgs e)
        {
            LeagueForm leagueForm = new LeagueForm(LeagueFormMode.New, mAppState);
            if (leagueForm.ShowDialog() == DialogResult.OK)
            {
                InitLeagues(leagueForm.SelectedLeagueName, leagueForm.SelectedSeason);
            }
        }

        /// <summary>
        /// Delete a league.
        /// </summary>
        private void MenuItemDeleteLeague_Click(object sender, System.EventArgs e)
        {
            DeleteCurrentLeague();
        }

        /// <summary>
        /// Edit the current league.
        /// </summary>
        private void MenuItemEditLeague_Click(object sender, System.EventArgs e)
        {
            LeagueForm leagueForm = new LeagueForm(LeagueFormMode.Edit, mAppState);
            if (leagueForm.ShowDialog() == DialogResult.OK)
            {
                // Invalidate data on pages
                mTabPageTable.InvalidateData();
                mTabPageTeams.InvalidateData();
                mTabPageLeague.InvalidateData();
                
                // This call makes sure league and season combos are updated
                InitLeagues(leagueForm.SelectedLeagueName, leagueForm.SelectedSeason);
            }
        }

        /// <summary>
        /// Edit teams.
        /// </summary>
        private void MenuItemEditTeams_Click(object sender, System.EventArgs e)
        {
            new TeamForm(mAppState).ShowDialog();

            // Invalidate data on pages
            mTabPageTable.InvalidateData();
            mTabPageMatches.InvalidateData();
            mTabPageLeague.InvalidateData();

            // Team page needs to be updated to reflect team name changes.
            mTabPageTeams.InitPage();
        }

        /// <summary>
        /// Edit fixtures.
        /// </summary>
        private void MenuItemEditFixtures_Click(object sender, System.EventArgs e)
        {
            new FixtureForm(mAppState).ShowDialog();

            // Invalidate data on pages
            mTabPageTable.InvalidateData();
            mTabPageTeams.InvalidateData();
            mTabPageLeague.InvalidateData();

            // Since the current round may have changed we need to update the match page
            mTabPageMatches.InitPage();
        }

        /// <summary>
        /// League order.
        /// </summary>
        private void MenuItemLeagueOrder_Click(object sender, System.EventArgs e)
        {
            new LeagueOrderForm(mAppState).ShowDialog();
            InitLeagues();
        }

        /// <summary>
        /// Exit application.
        /// </summary>
        private void MenuItemExit_Click(object sender, System.EventArgs e)
        {
            // Exit application by closing the main form
            this.Close();
        }

        /// <summary>
        /// Open Select-leagues dialog.
        /// </summary>
        private void MenuItemSelectLeagues_Click(object sender, System.EventArgs e)
        {
            new LiveUpdateSelectForm().ShowDialog();
        }

        /// <summary>
        /// Download selected leagues.
        /// </summary>
        private void MenuItemDownloadSelected_Click(object sender, System.EventArgs e)
        {
            DownloadSelectedLeagues();
        }

        /// <summary>
        /// Download current league.
        /// </summary>
        private void MenuItemDownloadThisLeague_Click(object sender, System.EventArgs e)
        {
            LiveUpdateDownloadForm downloadForm = new LiveUpdateDownloadForm(mAppState.CurrentLeague);
            downloadForm.ShowDialog();
            HandleLiveUpdateDialogResult(downloadForm.Result);
        }

        /// <summary>
        /// Download leagues from archive.
        /// </summary>
        private void MenuItemDownloadArchive_Click(object sender, System.EventArgs e)
        {
            LiveUpdateArchiveForm archiveForm = new LiveUpdateArchiveForm();
            if ((archiveForm.ShowDialog() == DialogResult.OK) &&
                (archiveForm.SelectedLeagueList.Count > 0))
            {
                LiveUpdateDownloadForm msgForm = new LiveUpdateDownloadForm(archiveForm.SelectedLeagueList, false);
                msgForm.ShowDialog();
                HandleLiveUpdateDialogResult(msgForm.Result);
            }
        }

        /// <summary>
        /// Display LiveUpdate log.
        /// </summary>
        private void MenuItemLog_Click(object sender, System.EventArgs e)
        {
            new LiveUpdateLogForm().ShowDialog();
        }
        
        /// <summary>
        /// About menu.
        /// </summary>
        private void MenuItemAbout_Click(object sender, System.EventArgs e)
        {
            new AboutForm().ShowDialog();
        }
               
        /// <summary>
        /// The LiveUpdate toolbar-button is clicked.
        /// </summary>
        private void ToolBar_ButtonClick(object sender, ToolBarButtonClickEventArgs e)
        {
            DownloadSelectedLeagues();
        }

        #endregion
        
        /// <summary>
        /// Shows different GUI's depending on if at least one
        /// league file was found or not.
        /// </summary>
        private void SetUIMode(UIMode mode)
        {            
            mMenuItemFile.MenuItems.Clear();
            mMenuItemLiveUpdate.MenuItems.Clear();

            if (mode == UIMode.Normal)
            {
                mTabControlMain.Show();
                mComboBoxLeague.Show();
                mComboBoxSeason.Show();
                if (mLabelNoLeagues != null)
                {
                    mLabelNoLeagues.Hide();
                }

                mMenuItemFile.MenuItems.Add(mMenuItemNewLeague);
                mMenuItemFile.MenuItems.Add(mMenuItemDeleteLeague);
                mMenuItemFile.MenuItems.Add(mMenuItemEdit);
                mMenuItemFile.MenuItems.Add(mMenuItemSep1);
                mMenuItemFile.MenuItems.Add(mMenuItemLeagueOrder);
                mMenuItemFile.MenuItems.Add(mMenuItemSep2);
                mMenuItemFile.MenuItems.Add(mMenuItemExit);

                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemSelectLeagues);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemDownloadSelected);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemSep3);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemDownloadThisLeague);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemDownloadArchive);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemSep4);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemLog);
            }
            else // mode == UIMode.NoLeagues
            {
                mTabControlMain.Hide();
                mComboBoxLeague.Hide();
                mComboBoxSeason.Hide();

                // Create label for No League message
                if (mLabelNoLeagues == null)
                {
                    mLabelNoLeagues = new System.Windows.Forms.Label();
                    mLabelNoLeagues.Parent = this;
                    mLabelNoLeagues.Bounds = new Rectangle(8, 32, 224, 24);
                    mLabelNoLeagues.Text = Language.GetString("mainform", "noleaguesfound");
                    mLabelNoLeagues.TextAlign = System.Drawing.ContentAlignment.TopCenter;
                }
                mLabelNoLeagues.Show();

                mMenuItemFile.MenuItems.Add(mMenuItemNewLeague);
                mMenuItemFile.MenuItems.Add(mMenuItemSep1);
                mMenuItemFile.MenuItems.Add(mMenuItemExit);

                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemSelectLeagues);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemDownloadSelected);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemSep3);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemDownloadArchive);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemSep4);
                mMenuItemLiveUpdate.MenuItems.Add(mMenuItemLog);
            }
        }

        /// <summary>
        /// Initilaizes the application by looking for league
        /// files and preparing controls.
        /// </summary>
        private void InitLeagues()
        {
            InitLeagues(null, null);
        }

        /// <summary>
        /// Initializes the application by looking for league
        /// files and preparing controls.
        /// Displays the league specified by leagueName and season.
        /// </summary>
        private void InitLeagues(string leagueName, string season)
        {
            // Search for league files
            LookForLeagues();

            // If no league found -> display empty screen
            if (mLeagues.Count == 0)
            {
                SetUIMode(UIMode.NoLeagues);
            }
            else
            {
                SetUIMode(UIMode.Normal);
                FillLeagueCombo();
                // No league specified -> show first league
                if (leagueName == null)
                {
                    mComboBoxLeague.SelectedIndex = 0;
                }
                // Show specified league and season
                else
                {
                    if (mComboBoxLeague.Items.Contains(leagueName))
                    {
                        mComboBoxLeague.SelectedItem = leagueName;
                        if (mComboBoxSeason.Items.Contains(season))
                        {
                            mComboBoxSeason.SelectedItem = season;
                        }
                    }
                    else
                    {
                        mComboBoxLeague.SelectedIndex = 0;
                    }
                }
            }
        }

        /// <summary>
        /// Called when a new league or season is selected. 
        /// Checks the combo boxes and redraws the screen.
        /// </summary>
        private void ShowLeague()
        {
            if (mAppState.Mode == LeagueViewMode.Matches)
            {
                // Save any pending changes on the match screen
                mTabPageMatches.SaveChangedResults();
            }

            string leagueName = (string)mComboBoxLeague.SelectedItem;
            string seasonName = (string)mComboBoxSeason.SelectedItem;

            ListDictionary seasons      = (ListDictionary)mLeagues[leagueName];
            mAppState.CurrentFilename   = (string)seasons[seasonName];
            mAppState.CurrentLeagueName = leagueName;
            mAppState.CurrentSeason     = seasonName;

            // Intialize controls on the pages to reflect the new league
            mTabPageTable.InitPage();
            mTabPageMatches.InitPage();
            mTabPageTeams.InitPage();
            mTabPageLeague.InitPage();

            // Redraw the screen
            mTabControlMain.Controls[mTabControlMain.SelectedIndex].Invalidate();
        }


        /// <summary>
        /// Fills the combo box with seasons for the selected league.
        /// </summary>
        private void FillSeasonCombo()
        {
            // Get the currently selected league name
            string leagueName = (string)mComboBoxLeague.SelectedItem;
            // Get ListDictionary with seasons for the selected league
            ListDictionary seasons = (ListDictionary)mLeagues[leagueName];
            
            // Sort the seasons (this is not very efficient but the number 
            // of seasons will likely be small)
            string[] arr = new string[seasons.Count];
            seasons.Keys.CopyTo(arr, 0);
            Array.Sort(arr);
            Array.Reverse(arr, 0, arr.Length);

            // Add items to combo
            mComboBoxSeason.Items.Clear();
            foreach (string s in arr)
            {
                mComboBoxSeason.Items.Add(s);
            }

        }

        /// <summary>
        /// Fills the combo box with league names.
        /// </summary>
        private void FillLeagueCombo()
        {
            mComboBoxLeague.Items.Clear();

            // This loop copies the league names in the ListDictionary mLeagues
            // to the combo box. The league names should appear in the combo box
            // in the order specified in AppState.LeagueNames. For each league
            // in mLeagues we need to loop through AppState.LeagueNames to find
            // the correct position.
            foreach (DictionaryEntry de in mLeagues)
            {
                string leagueName = (string)de.Key;
                // If combobox is empty -> just add the league
                if (mComboBoxLeague.Items.Count == 0)
                {
                    mComboBoxLeague.Items.Add(leagueName);
                }
                else
                {
                    int listBoxPos = 0;
                    int i;
                    for (i = 0; i < mAppState.LeagueNames.Length; i++)
                    {
                        // Correct position found -> add league and break out of loop
                        if (mAppState.LeagueNames[i] == leagueName)
                        {
                            mComboBoxLeague.Items.Insert(listBoxPos, leagueName);
                            break;
                        }
                        // Another league was found -> increment listboxpos
                        else if ((listBoxPos < mComboBoxLeague.Items.Count) &&
                                 (mAppState.LeagueNames[i] == (string)mComboBoxLeague.Items[listBoxPos]))
                        {
                            listBoxPos++;
                        }
                    }

                    // The league was not found in AppState.LeagueNames -> add it at the bottom
                    if (i == mAppState.LeagueNames.Length)
                    {
                        mComboBoxLeague.Items.Add(leagueName);
                    }
                }
            }

            // Finally we copy all items from the combo box to appstate
            // to make sure they are equal
            mAppState.LeagueNames = new string[mComboBoxLeague.Items.Count];
            for (int i = 0; i < mComboBoxLeague.Items.Count; i++)
            {
                mAppState.LeagueNames[i] = (string)mComboBoxLeague.Items[i];
            }
        }

        /// <summary>
        /// Adds information about a found league file.
        /// </summary>
        private void AddLeague(FileInfo file) 
        {            
            LeagueBinaryFile leagueFile;
            League league;
            try
            {
                leagueFile = new LeagueBinaryFile(file.FullName);
                league = new League(leagueFile);
            }
            catch (System.IO.IOException)
            {
                // Bail out and ignore this file
                return;
            }
            
            // Look for previous seasons of this league
            ListDictionary seasons = (ListDictionary)mLeagues[league.Name];
            // If no prev. seasons found -> create new hashtable
            if (seasons == null)
            {
                seasons = new ListDictionary();
                mLeagues.Add(league.Name, seasons);
            }

            // Store info about this file
            if (!seasons.Contains(league.Season))
            {
                seasons.Add(league.Season, file.FullName);
            }
        }

        /// <summary>
        /// Scans the application directory and loads all league
        /// files it finds.
        /// </summary>
        private void LookForLeagues()
        {
            // Clear the ListDictionary
            mLeagues = new ListDictionary();

            // Create a reference to the applications directory
            DirectoryInfo dirInfo = new DirectoryInfo(Utils.AppDirectory);
            // Create an array representing the files files in the directory
            FileInfo[] files = dirInfo.GetFiles("*.ts");

            // Add the files
            foreach (FileInfo fileInfo in files)
            {
                AddLeague(fileInfo);
            }
            
        }

        /// <summary>
        /// Deletes the currently active league file.
        /// </summary>
        private void DeleteCurrentLeague()
        {
            League league = mAppState.CurrentLeague;

            // Show confirmation dialog
            DialogResult dialogResult = MessageBox.Show(
                Language.GetString("mainform", "reallydelete") + league.FullName + "?", 
                Language.GetString("mainform", "deleteleague"), 
                MessageBoxButtons.YesNo,
                MessageBoxIcon.Question,
                MessageBoxDefaultButton.Button2);

            // Delete the file and re-initialize app
            if (dialogResult == DialogResult.Yes)
            {
                File.Delete(mAppState.CurrentFilename);
                InitLeagues();
            }
        }

        /// <summary>
        /// Downloads selected leagues.
        /// </summary>
        private void DownloadSelectedLeagues()
        {
            // Download leagues if any are selected...
            if (mAppState.DownloadLeagueList.Count > 0)
            {
                // Create and show message form
                LiveUpdateDownloadForm downloadForm = 
                    new LiveUpdateDownloadForm(mAppState.DownloadLeagueList, true);
                downloadForm.ShowDialog();
                HandleLiveUpdateDialogResult(downloadForm.Result);
            }
            // ...otherwise display info about LiveUpdate
            else
            {
                MessageBox.Show(
                    Language.GetString("liveupdate", "noleagues"), 
                    Language.GetString("liveupdatedownloadform", "title"));
            }
        }

        /// <summary>
        /// Handles the result of a LiveUpdate-dialog
        /// </summary>
        private void HandleLiveUpdateDialogResult(LiveUpdateDownloadForm.FormResult result)
        {
            if (result == LiveUpdateDownloadForm.FormResult.Ok)
            {
                // League were downloaded -> re-init leagues
                InitLeagues((string)mComboBoxLeague.SelectedItem, (string)mComboBoxSeason.SelectedItem);
            }
            else if (result == LiveUpdateDownloadForm.FormResult.NoConnection)
            {
                // Display "Not connected"-message
                MessageBox.Show(
                    Language.GetString("liveupdate", "noconnection"), 
                    Language.GetString("liveupdatedownloadform", "title"));
            }
            else if (result == LiveUpdateDownloadForm.FormResult.LeagueNotFound)
            {
                // League not found
                MessageBox.Show(
                    Language.GetString("liveupdate", "leaguenotavailable"), 
                    Language.GetString("liveupdatedownloadform", "title"));
            }
        }

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        public static void Main() 
        {
            Application.Run(new MainForm());
        }

    }
}
