/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Drawing;
using System.Collections;
using System.Threading;
using System.Windows.Forms;
using System.Data;
using MBovin.TeamStats.Util;
using MBovin.TeamStats.LiveUpdate;

namespace MBovin.TeamStats.UI
{
    /// <summary>
    /// Form for selecting leagues to download. This form will present a tree
    /// view of all the leagues that are available for download.
    /// </summary>
    public class LiveUpdateArchiveForm : System.Windows.Forms.Form
    {
        #region Support types
        private enum CheckState
        {
            Fully,
            Partly,
            None
        }
        #endregion

        private Label    mLabelLeagueCount;
        private TextBox  mTextBoxInfo;
        private TreeView mTreeView;

        private IList mSelectedLeagueList = new ArrayList();
        private int   mCheckedCount = 0;


        /// <summary>
        /// Constructor
        /// </summary>
        public LiveUpdateArchiveForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// List with selected leagues. The list contains SoccerDB groupId:s.
        /// </summary>
        public IList SelectedLeagueList
        {
            get { return mSelectedLeagueList; }
        }

        /// <summary>
        /// Required method for Designer support - do not modify
        /// the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            int screenHeight = Screen.PrimaryScreen.WorkingArea.Height;

            Label topLabel  = new Label();
            topLabel.Parent = this;
            topLabel.Bounds = new Rectangle(8, 4, 224, 15);
            topLabel.Font   = new Font("Tahoma", 8F, FontStyle.Regular);
            topLabel.Text   = Language.GetString("liveupdatearchiveform", "selectleagueslabel");

            mLabelLeagueCount        = new Label();
            mLabelLeagueCount.Parent = this;
            mLabelLeagueCount.Bounds = new Rectangle(8, 19, 224, 15);
            mLabelLeagueCount.Font   = new Font("Tahoma", 8F, FontStyle.Regular);
            mLabelLeagueCount.Text   = string.Format(
                Language.GetString("liveupdatearchiveform", "leaguecountlabel"), 0);

            mTextBoxInfo           = new TextBox();
            mTextBoxInfo.Parent    = this;
            mTextBoxInfo.Bounds    = new Rectangle(8, 40, 224, screenHeight - 84);
            mTextBoxInfo.Multiline = true;
            mTextBoxInfo.Text      = Language.GetString("liveupdatearchiveform", "downloadinglist");

            mTreeView            = new TreeView();
            mTreeView.Parent     = this;
            mTreeView.CheckBoxes = true;
            mTreeView.Bounds     = new Rectangle(8, 40, 224, screenHeight - 84);
            mTreeView.Visible    = false;
            mTreeView.AfterCheck += new TreeViewEventHandler(TreeView_AfterCheck);

            Button buttonDownload       = new Button();
            buttonDownload.Parent       = this;
            buttonDownload.Bounds       = new Rectangle(8, screenHeight - 34, 100, 20);
            buttonDownload.Font         = MainForm.cButtonFont;
            buttonDownload.Text         = Language.GetString("liveupdatearchiveform", "download");
            buttonDownload.DialogResult = DialogResult.OK;
            buttonDownload.Click += new EventHandler(ButtonDownload_Click);

            Button buttonCancel       = new Button();
            buttonCancel.Parent       = this;
            buttonCancel.Bounds       = new Rectangle(116, screenHeight - 34, 60, 20);
            buttonCancel.Font         = MainForm.cButtonFont;
            buttonCancel.Text         = Language.GetString("liveupdatearchiveform", "cancel");
            buttonCancel.DialogResult = DialogResult.Cancel;

            this.Text = Language.GetString("liveupdatearchiveform", "title");
        }

        /// <summary>
        /// Fires a  thread that will download the list with leagues.
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            // Start a thread that will try to download leagues from the net
            Thread thread = new Thread(new ThreadStart(this.LoadLeagueList));
            thread.Start();
        }

        /// <summary>
        /// Downloads a list with leagues and adds nodes to the TreeView-control.
        /// </summary>
        private void LoadLeagueList()
        {
            LeagueListManager leagueManager = new LeagueListManager();
            
            if (leagueManager.IsLocalArchiveListAvailable() || 
                new ConnectionTester().IsConnectionAvailable())
            {
                // Get the list with leagues
                IList leagueList = leagueManager.GetArchiveList();
                TreeNode[] treeNodeArr =
                    (TreeNode[])(((ArrayList)leagueList).ToArray(new TreeNode().GetType()));

                // Update UI
                mTreeView.Nodes.AddRange(treeNodeArr);
                mTextBoxInfo.Visible = false;
                mTreeView.Visible    = true;
            }
            else // No connection available - display message in text box
            {
                mTextBoxInfo.Invoke(new EventHandler(SetTextNoConnection));
            }
        }

        /// <summary>
        /// Handler for when a checkbox is checked or unchecked.
        /// </summary>
        private void TreeView_AfterCheck(object sender, TreeViewEventArgs e)
        {
            if (e.Action != TreeViewAction.Unknown)
            {
                // If the node has children we should change their check recursively
                if (e.Node.Nodes.Count > 0)
                {
                    CheckAllChildNodes(e.Node, e.Node.Checked);
                }
                else // If it's a leaf node we should update the count of checked nodes.
                {
                    mCheckedCount += (e.Node.Checked ? 1 : -1);
                }

                // Get the top node of this node
                TreeNode topNode = e.Node;
                while (topNode.Parent != null)
                {
                    topNode = topNode.Parent;
                }

                // Set the triple-check-state (fully checked, partly checked, or unchecked)
                // of the entire branch.
                SetTripleCheckState(topNode);
            }

            mLabelLeagueCount.Text = string.Format(
                Language.GetString("liveupdatearchiveform", "leaguecountlabel"), mCheckedCount);
        }

        /// <summary>
        /// Sets the check state of all the children og the specified node.
        /// </summary>
        /// <param name="treeNode">A node</param>
        /// <param name="newChecked">The new check state</param>
        private void CheckAllChildNodes(TreeNode treeNode, bool newChecked)
        {
            // Loop over children
            foreach (TreeNode node in treeNode.Nodes)
            {
                // Current check state
                bool oldChecked = node.Checked;
                
                // Set check state
                node.Checked = newChecked;

                if (node.Nodes.Count > 0)
                {
                    // If the current node has child nodes, 
                    // call the CheckAllChildsNodes method recursively.
                    CheckAllChildNodes(node, newChecked);
                }
                else // This is a leaf node
                {
                    // If the check stated changed we need to update
                    // the current checked count.
                    if (oldChecked && !newChecked) mCheckedCount--;
                    if (!oldChecked && newChecked) mCheckedCount++;
                }
            }
        }

        /// <summary>
        /// Sets the triple-check-state (fully checked, partly checked, or unchecked)
        /// of the specified node and all its children.
        /// </summary>
        /// <param name="node">A node</param>
        /// <returns>The triple-check-state of the specified node</returns>
        private CheckState SetTripleCheckState(TreeNode node)
        {
            CheckState checkState;

            if (node.Nodes.Count > 0)
            {
                int childFullyCheckedCount = 0;
                int childNotCheckedCount   = 0;
                
                // Loop over children
                foreach (TreeNode child in node.Nodes)
                {
                    CheckState childCheckState = SetTripleCheckState(child);
                    childFullyCheckedCount += (childCheckState == CheckState.Fully ? 1 : 0);
                    childNotCheckedCount   += (childCheckState == CheckState.None  ? 1 : 0);
                }

                // Determine the check state of this node
                if (childFullyCheckedCount == node.Nodes.Count)
                {
                    checkState = CheckState.Fully;
                }
                else if (childNotCheckedCount == node.Nodes.Count)
                {
                    checkState = CheckState.None;
                }
                else
                {
                    checkState = CheckState.Partly;
                }
            }
            else // This is a leaf node
            {
                checkState = node.Checked ? CheckState.Fully : CheckState.None;
            }

            // Update UI to reflect check state
            switch (checkState)
            {
                case CheckState.None :
                    node.Checked = false;
                    if (node.Text.StartsWith("*"))
                    {
                        node.Text = node.Text.Substring(1);
                    }
                    break;

                case CheckState.Partly :
                    node.Checked = false;
                    if (!node.Text.StartsWith("*"))
                    {
                        node.Text = "*" + node.Text;
                    }
                    break;

                case CheckState.Fully:
                    node.Checked = true;
                    if (!node.Text.StartsWith("*"))
                    {
                        node.Text = "*" + node.Text;
                    }
                    break;
            }

            return checkState;
        }

        /// <summary>
        /// Handler for download button.
        /// </summary>
        private void ButtonDownload_Click(object sender, EventArgs e)
        {
            if (mTreeView.Visible)
            {
                foreach (TreeNode node in mTreeView.Nodes)
                {
                    PickSelectedLeagues(node);
                }
            }
        }

        /// <summary>
        /// Loops over all the nodes and adds checked leaf nodes 
        /// to the list with selected leagues.
        /// </summary>
        /// <param name="node">A node</param>
        private void PickSelectedLeagues(TreeNode node)
        {
            foreach (TreeNode child in node.Nodes)
            {
                if ((child.Tag != null) && child.Checked)
                {
                    mSelectedLeagueList.Add((int)child.Tag);
                }
                PickSelectedLeagues(child);
            }
        }

        /// <summary>
        /// Displays a "No connection"-message in the text box
        /// </summary>
        private void SetTextNoConnection(object sender, EventArgs e)
        {
            mTextBoxInfo.Text = Language.GetString("liveupdatearchiveform", "noconnection");
        }

    }
}
