/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using MBovin.TeamStats.Core;
using MBovin.TeamStats.LiveUpdate;
using MBovin.TeamStats.Util;

namespace MBovin.TeamStats.UI
{
	/// <summary>
	/// Form that runs LiveUpdate. 
	/// </summary>
	public class LiveUpdateDownloadForm : System.Windows.Forms.Form
    {
        #region Support types
        /// <summary>
        /// Enum for different modes this form can run in.
        /// </summary>
        private enum FormMode
        {
            DownloadSingle,
            DownloadList
        }

        /// <summary>
        /// Enum for the result of the dialog.
        /// </summary>
        public enum FormResult
        {
            Ok,
            NoConnection,
            LeagueNotFound
        }
        #endregion

        // Controls
        private Label       mStatusLabel;
        private ProgressBar mProgressBar;
        private Button      mButtonCancel;

        private FormMode   mMode;
        private FormResult mFormResult = FormResult.Ok;

        private League mLeague;
        private IList  mLeagueList;
        private bool   mCheckForNewLeagues;

        private LeagueDownloader  mLeagueDownloader;
        private LeagueListManager mLeagueListManager;

        private int  mDownloadedLeagueCount = 0;
        private bool mIsAsyncRunning;
        private bool mIsCancelPending;


        /// <summary>
        /// Constructor
        /// </summary>
		public LiveUpdateDownloadForm() : this(null) {}

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="league">League to download</param>
		public LiveUpdateDownloadForm(League league)
		{
            mLeague = league;
            mCheckForNewLeagues = false;
            Init();
		}

        /// <summary>
        /// Constuctor
        /// </summary>
        /// <param name="leagueList">List with leagues to download</param>
        /// <param name="checkForNewLeagues">Set to true to also check for new leagues</param>
        public LiveUpdateDownloadForm(IList leagueList, bool checkForNewLeagues)
        {
            mLeagueList = leagueList;
            mCheckForNewLeagues = checkForNewLeagues;
            Init();
        }

        /// <summary>
        /// Initializes the object.
        /// </summary>
        private void Init()
        {
            mMode = (mLeague != null) ? FormMode.DownloadSingle : FormMode.DownloadList;

            // Create a LeagueDownloader and register for events
            mLeagueDownloader = new LeagueDownloader();
            mLeagueDownloader.LiveUpdateStateChanged +=
                new LiveUpdateEventHandler(this.LeagueDownloader_DownloadStateChanged);
            mLeagueDownloader.Log += new LogEventHandler(AppState.Instance.LiveUpdate_Log);

            if (mCheckForNewLeagues)
            {
                // Create a LeagueListManager and register for events
                mLeagueListManager = new LeagueListManager();
                mLeagueListManager.LiveUpdateStateChanged +=
                    new LiveUpdateEventHandler(this.LeagueDownloader_DownloadStateChanged);
                mLeagueListManager.Log += new LogEventHandler(AppState.Instance.LiveUpdate_Log);
            }

            InitializeComponent();
        }

		/// <summary>
        /// Initializes GUI.
        /// </summary>
        private void InitializeComponent()
		{
            // StatusLabel
            mStatusLabel        = new Label();
            mStatusLabel.Parent = this;
            mStatusLabel.Bounds = new Rectangle(50, 20, 140, 35);

            // ProgressBar
            mProgressBar         = new ProgressBar();
            mProgressBar.Parent  = this;
            mProgressBar.Bounds  = new Rectangle(50, 55, 140, 20);
            mProgressBar.Visible = false;

            // Cancel button
            mButtonCancel              = new Button();
            mButtonCancel.Parent       = this;
            mButtonCancel.Text         = Language.GetString("liveupdatedownloadform", "cancel");
            mButtonCancel.Bounds       = new Rectangle(70, 90, 60, 20);
            mButtonCancel.Font         = MainForm.cButtonFont;
            mButtonCancel.DialogResult = DialogResult.Cancel;

            // Form
            this.Text            = Language.GetString("liveupdatedownloadform", "title");
            this.FormBorderStyle = FormBorderStyle.None;
            this.Bounds          = new Rectangle(20, 40, 200, 120);
		}

        /// <summary>
        /// Returns the result after this form has been displayed.
        /// </summary>
        public FormResult Result
        {
            get { return mFormResult; }
        }

        /// <summary>
        /// The number of leagues to download.
        /// </summary>
        private int LeagueCount
        {
            get 
            {
                return (mMode == FormMode.DownloadList) ? mLeagueList.Count : 1;
            }
        }

        /// <summary>
        /// Paints the form.
        /// </summary>
        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            // Draw a border around the form
            e.Graphics.DrawRectangle(new Pen(Color.Black), 0, 0, this.Width-1, this.Height-1);
            // Draw the LiveUpdate icon
            e.Graphics.DrawIcon(Utils.LoadEmbeddedIcon("MBovin.TeamStats.LiveUpdate.ico", 32, 32), 10, 20);
        }

        /// <summary>
        /// Form is loaded.
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            // Reset the log
            AppState.Instance.LiveUpdateLog = Language.GetString("liveupdate", "started") +
                DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToLongTimeString() + "\r\n\r\n";

            // Fires thread that tests the connection
            TestConnection();
        }

        /// <summary>
        /// We only let the form close if no asynchronous activities are going on.
        /// </summary>
        protected override void OnClosing(CancelEventArgs e)
        {
            if (mIsAsyncRunning)
            {
                // Tell LeagueDownloader to stop
                mLeagueDownloader.CancelAsyncDownload();
                mIsCancelPending      = true;
                mButtonCancel.Enabled = false;
                mStatusLabel.Text     = Language.GetString("liveupdatedownloadform", "canceling");
                mProgressBar.Visible  = false;
                // This will cancel the close-event
                e.Cancel = true;
            }
            else
            {
                 AppState.Instance.LiveUpdateLog += ("\r\n" + Language.GetString("liveupdate", "finished") +
                    DateTime.Now.ToShortDateString() + " " + DateTime.Now.ToLongTimeString());
            }
            base.OnClosing(e);
        }

        /// <summary>
        /// Starts thread that tests connectivity.
        /// </summary>
        private void TestConnection()
        {
            mStatusLabel.Text = Language.GetString("liveupdatedownloadform", "connecting");
            ConnectionTester tester = new ConnectionTester();
            tester.Log += new LogEventHandler(AppState.Instance.LiveUpdate_Log);
            tester.LiveUpdateStateChanged += new LiveUpdateEventHandler(this.LeagueDownloader_DownloadStateChanged);
            tester.TestConnectionAsync();
            mIsAsyncRunning = true;
        }

        /// <summary>
        /// Starts checking for new leagues.
        /// </summary>
        private void CheckForNewLeagues()
        {
            mStatusLabel.Text = Language.GetString("liveupdatedownloadform", "checkingnew");
            mLeagueListManager.CheckForNewLeaguesAsync();
            mIsAsyncRunning = true;
        }

        /// <summary>
        /// Starts downloading leagues.
        /// </summary>
        private void DownloadLeagues(object sender, EventArgs e)
        {
            mStatusLabel.Text = 
                Language.GetString("liveupdatedownloadform", "downloading") + "\r\n" +
                string.Format(
                    Language.GetString("liveupdatedownloadform", "remaining"), this.LeagueCount);
            mProgressBar.Minimum = 0;
            mProgressBar.Maximum = this.LeagueCount;
            mProgressBar.Value   = 0;
            mProgressBar.Visible = true;

            mLeagueDownloader.DownloadLeaguesAsync(mLeagueList);
            mIsAsyncRunning = true;
        }

        /// <summary>
        /// Downloads a single league.
        /// </summary>
        private void DownloadSingleLeague()
        {
            mStatusLabel.Text = 
                Language.GetString("liveupdatedownloadform", "downloading") + "\r\n" +
                string.Format(
                    Language.GetString("liveupdatedownloadform", "remaining"), 1);
            mProgressBar.Minimum = 0;
            mProgressBar.Maximum = 1;
            mProgressBar.Value   = 0;
            mProgressBar.Visible = true;

            mLeagueDownloader.DownloadLeagueAsync(mLeague);
            mIsAsyncRunning = true;
        }

        /// <summary>
        /// Displays form with newly added league where the user can 
        /// select which leagues to add.
        /// </summary>
        private void HandleNewLeagues()
        {
            if (mLeagueListManager.AddedLeagueList.Count > 0)
            {
                Invoke(new EventHandler(ShowNewLeaguesDialog));
            }
        }

        /// <summary>
        /// Displays the select-new-league-dialog
        /// </summary>
        private void ShowNewLeaguesDialog(object sender, EventArgs e)
        {
            new LiveUpdateSelectForm(mLeagueListManager.AddedLeagueList).ShowDialog();
        }

        /// <summary>
        /// Handles state changes during the liveupdate process.
        /// </summary>
        public void LeagueDownloader_DownloadStateChanged(object sender, LiveUpdateEvent liveUpdateEvent)
        {
            switch (liveUpdateEvent)
            {
                // No connection was available -> close the form
                case LiveUpdateEvent.NoConnection:
                    mIsAsyncRunning = false;
                    mFormResult     = FormResult.NoConnection;
                    Close();
                    break;

                // A connection was available
                case LiveUpdateEvent.ConnectionOk :
                    mIsAsyncRunning = false;
                    
                    if (!mIsCancelPending) 
                    {
                        // Connection ok -> start next action
                        switch (mMode) 
                        {
                            case FormMode.DownloadList :
                                if (mCheckForNewLeagues)
                                {
                                    CheckForNewLeagues();
                                }
                                else
                                {
                                    DownloadLeagues(null, null);
                                }
                                break;
                            case FormMode.DownloadSingle :
                                DownloadSingleLeague();
                                break;
                        }
                    }
                    else // Cancel pending -> close form
                    {
                        Invoke(new EventHandler(CloseForm));
                    }
                    break;

                // Checking for new leagues is done
                case LiveUpdateEvent.CheckForNewLeaguesFinished:
                    mIsAsyncRunning = false;
                    if (!mIsCancelPending)
                    {
                        // Handle any new league and then start downloading
                        HandleNewLeagues();
                        Invoke(new EventHandler(DownloadLeagues));
                    }
                    else // Cancel pending -> close form
                    {
                        Invoke(new EventHandler(CloseForm));
                    }
                    break;

                // A league has finished downloading
                case LiveUpdateEvent.LeagueDownloaded:                    
                    // Update log and progress bar
                    mDownloadedLeagueCount++;
                    mStatusLabel.Text =
                        Language.GetString("liveupdatedownloadform", "downloading") + "\r\n" +
                        string.Format(
                            Language.GetString("liveupdatedownloadform", "remaining"), 
                                this.LeagueCount - mDownloadedLeagueCount);
                    mProgressBar.Value = mDownloadedLeagueCount;
                    break;

                // A league was not available on the server
                case LiveUpdateEvent.LeagueNotFound :
                    mFormResult = FormResult.LeagueNotFound;
                    break;

                // Downloading of leagues is done.
                case LiveUpdateEvent.Finished:
                    mIsAsyncRunning   = false;
                    mStatusLabel.Text = Language.GetString("liveupdatedownloadform", "done");
                    Invoke(new EventHandler(CloseForm));
                    break;
            }
        }

        /// <summary>
        /// Closes the form.
        /// </summary>
        private void CloseForm(object sender, EventArgs e)
        {
            Close();
        }
	}
}
