/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Threading;
using System.Windows.Forms;
using MBovin.TeamStats.Controls;
using MBovin.TeamStats.LiveUpdate;
using MBovin.TeamStats.Util;

namespace MBovin.TeamStats.UI
{
	/// <summary>
	/// Form for selecting leagues for LiveUpdate. When this form is opened it
	/// will try to download a list with active leagues from the internet.
    /// Also used to display a list of newly added leagues where the user
    /// can select the leagues she wants to add.
	/// </summary>
    public class LiveUpdateSelectForm : System.Windows.Forms.Form
    {
        /// <summary>
        /// Enum for different modes this form can run in.
        /// </summary>
        private enum FormMode
        {
            SelectLeagues,
            AddNewLeagues
        }

        private ScrollPanel mScrollPanelLeagues;
        private TextBox     mTextBoxInfo;
        private CheckBox[]  mCheckBoxLeague = null;

        private FormMode mMode;
        private IList    mLeagueList;
        private int      mLeagueListIndex;


        /// <summary>
        /// Constructor.
        /// </summary>
        public LiveUpdateSelectForm() : this(null) {}

        /// <summary>
        /// Constructor.
        /// </summary>
        /// <param name="leagueList">A list with leagues to display, 
        /// instead of downloading the list</param>
        public LiveUpdateSelectForm(IList leagueList)
        {
            mLeagueList = leagueList;
            
            // Decide which mode we're running in.
            if (mLeagueList == null)
            {
                mMode = FormMode.SelectLeagues;
            }
            else
            {
                mMode = FormMode.AddNewLeagues;
            }
            InitializeComponent();
        }

        /// <summary>
        /// Initializes GUI.
        /// </summary>
        private void InitializeComponent()
        {
            int screenHeight = Screen.PrimaryScreen.WorkingArea.Height;

            Label labelHeaderLeagues  = new Label();
            labelHeaderLeagues.Parent = this;
            labelHeaderLeagues.Bounds = new Rectangle(8, 4, 224, 30);
            labelHeaderLeagues.Font   = new Font("Tahoma", 8F, FontStyle.Regular);
            labelHeaderLeagues.Text   = (mMode == FormMode.SelectLeagues) ?
                Language.GetString("liveupdateselectform", "selectleagueslabel") :
                Language.GetString("liveupdateselectform", "selectnewleagueslabel");
            
            mTextBoxInfo           = new TextBox();
            mTextBoxInfo.Parent    = this;
            mTextBoxInfo.Bounds    = new Rectangle(8, 34, 224, screenHeight - 90);
            mTextBoxInfo.Multiline = true;
            mTextBoxInfo.Text      = Language.GetString("liveupdateselectform", "downloadinglist");

            mScrollPanelLeagues         = new ScrollPanel();
            mScrollPanelLeagues.Parent  = this;
            mScrollPanelLeagues.Bounds  = new Rectangle(8, 34, 224, screenHeight-90);
            mScrollPanelLeagues.Visible = false;

            Button buttonClose = new Button();
            buttonClose.Parent = this;
            buttonClose.Bounds = new Rectangle(8, screenHeight-36, 60, 20);
            buttonClose.Font   = MainForm.cButtonFont;
            buttonClose.Text   = Language.GetString("liveupdateselectform", "close");
            buttonClose.DialogResult = DialogResult.OK;

            this.Text = Language.GetString("liveupdateselectform", "title");
        }

        /// <summary>
        /// Event handler for when form is loades.
        /// </summary>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (mMode == FormMode.SelectLeagues)
            {
                // Start a thread that will try to download leagues from the net
                Thread thread = new Thread(new ThreadStart(this.LoadLeagueList));
                thread.Start();
            }
            else
            {
                PopulateScrollPanel();
            }
        }

        /// <summary>
        /// Event handler for when form is about to close.
        /// </summary>
        protected override void OnClosing(CancelEventArgs e)
        {
            base.OnClosing(e);

            if (mScrollPanelLeagues.Visible)
            {
                AppState appState = AppState.Instance;

                // If we're in select-mode we clear the list first
                if (mMode == FormMode.SelectLeagues)
                {
                    appState.DownloadLeagueList.Clear();
                }

                // Loop over the checkboxes and add checked leagues
                for (int i = 0; i < mCheckBoxLeague.Length; i++)
                {
                    if (mCheckBoxLeague[i].Checked &&
                        !appState.DownloadLeagueList.Contains(((DownloadableLeague)mLeagueList[i]).Id))
                    {
                        appState.DownloadLeagueList.Add(((DownloadableLeague)mLeagueList[i]).Id);
                    }
                }
            }
        }

        /// <summary>
        /// Tries to download a list with active leagues from the server. 
        /// </summary>
        private void LoadLeagueList()
        {            
            LeagueListManager leagueManager = new LeagueListManager();
            if (leagueManager.IsLocalActiveListAvailable() || 
                new ConnectionTester().IsConnectionAvailable())
            {
                mLeagueList = leagueManager.GetActiveList();
                PopulateScrollPanel();
            }
            else // No connection available - display message in text box
            {
                mTextBoxInfo.Invoke(new EventHandler(SetTextNoConnection));
            }
        }

        /// <summary>
        /// Populates the scrollpanel with the leagues in mLeagueListIndex.
        /// Adds a checkbox to the scrollpanel for each found league.
        /// </summary>
        private void PopulateScrollPanel()
        {
            // Create array with checkboxes
            mCheckBoxLeague = new CheckBox[mLeagueList.Count];

            // Add each league to the scrollpanel
            for (mLeagueListIndex = 0; mLeagueListIndex < mLeagueList.Count; mLeagueListIndex++)
            {
                mScrollPanelLeagues.Invoke(new EventHandler(AddLeague));
            }

            mTextBoxInfo.Visible        = false;
            mScrollPanelLeagues.Visible = true;
        }

        /// <summary>
        /// Adds a league checkbox to the scrollpanel.
        /// </summary>
        private void AddLeague(object sender, EventArgs e)
        {
            AppState appState = AppState.Instance;

            // Get the league data
            DownloadableLeague league = (DownloadableLeague)mLeagueList[mLeagueListIndex];
            
            // Create a checkbox
            mCheckBoxLeague[mLeagueListIndex] = new CheckBox();
            CheckBox cb = mCheckBoxLeague[mLeagueListIndex];
            cb.Bounds   = new Rectangle(8, 2 + 18 * mLeagueListIndex, 200, 20);
            cb.Font     = new Font("Tahoma", 8F, FontStyle.Regular);
            cb.Text     = league.Country + " - " + league.Name;
            cb.Checked  = appState.DownloadLeagueList.Contains(league.Id);

            // Add checkbox to panel
            mScrollPanelLeagues.Controls.Add(cb);
        }

        /// <summary>
        /// Displays a "No connection"-message in the text box
        /// </summary>
        private void SetTextNoConnection(object sender, EventArgs e)
        {
            mTextBoxInfo.Text = Language.GetString("liveupdateselectform", "noconnection");
        }

    }
}
