/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using Microsoft.WindowsCE.Forms;
using MBovin.TeamStats;
using MBovin.TeamStats.Controls;
using MBovin.TeamStats.Core;
using MBovin.TeamStats.Util;

namespace MBovin.TeamStats.UI
{
	/// <summary>
	/// This class impelements the match page.
	/// </summary>
	public class MatchPage : PageBase
	{        
        // Constants
        private const int cTopOffset = 18;
        private const int cRowHeight = 17;

        // Controls
        private Label            mLabelRound;
        private Button           mButtonRoundPrev;
        private Button           mButtonRoundNext;
        private CheckBox         mCheckBoxEdit;
        private DateTimePicker[] mDateTimePickerMatchDate;
        private TextBox[]        mTextBoxHomeGoals;
        private TextBox[]        mTextBoxAwayGoals;
        private VScrollBar       mVScrollBar;
        private InputPanel       mSip;
        
        // Indicates if the form is in edit mode
        private bool mIsEditMode       = false;
        private bool mIsEditGuiBuilt   = false;
        private bool mIsDatePickerOpen = false;
        // How much to display before we need scrolling
        private int mMatchesPerPage;
        private int mMatchesPerPageEdit;
        // Position of scroll bar
        private int mScrollPos = 0;
        
        // The current round
        private int mRound;
        // The matches of the round currently being displayed
        private ArrayList mMatchList;

        /// <summary>
        /// Contructor
        /// </summary>
        public MatchPage(AppState appState) : base(appState) 
        {
            this.Text = Language.GetString("matchpage", "title");
        }

        #region Events

        /// <summary>
        /// One of the date pickers is opened.
        /// </summary>
        private void DateTimePickerMatchDate_DropDown(object sender, EventArgs e)
        {
            mIsDatePickerOpen = true;
        }

        /// <summary>
        /// A new date is selected in one of the dateTimePicker controls.
        /// </summary>
        private void DateTimePickerMatchDate_CloseUp(object sender, System.EventArgs e)
        {
            // The CloseUp-event sometimes fires twice so we use a flag to make sure
            // we only process the event once per DropDown-event.
            if (mIsDatePickerOpen)
            {
                DateTimePicker dateTimePicker = (DateTimePicker)sender;
                int   index = Convert.ToInt32(dateTimePicker.Name);
                Match match = (Match)mMatchList[index + mScrollPos];

                // Check that the date is actually changed before saving
                if (match.Date != dateTimePicker.Value)
                {
                    match.Date = dateTimePicker.Value;
                    match.Save();
                    League league = mAppState.CurrentLeague;
                    league.SortMatches();
                    InvalidateData();
                    Invalidate();
                }

                mIsDatePickerOpen = false;
            }
        }

        /// <summary>
        /// The next round button is clicked.
        /// </summary>
        private void ButtonRoundNext_Click(object sender, System.EventArgs e)
        {
            League league = mAppState.CurrentLeague;
            if (mRound < league.RoundCount)
            {
                SaveChangedResults();
                mRound++;
                InvalidateData();
                Invalidate();
            }
        }

        /// <summary>
        /// The previous round button is clicked.
        /// </summary>
        private void ButtonRoundPrev_Click(object sender, System.EventArgs e)
        {
            if (mRound > 1)
            {
                SaveChangedResults();
                mRound--;
                InvalidateData();
                Invalidate();
            }
        }
        
        /// <summary>
        /// The scroll bar is used.
        /// </summary>
        private void VScrollBar_ValueChanged(object sender, System.EventArgs e)
        {
            SaveChangedResults();
            mScrollPos = mVScrollBar.Value;
            UpdateControls();
            Invalidate();
        }

        /// <summary>
        /// Handler for the Edit-checkbox.
        /// </summary>
        private void CheckBoxEdit_Click(object sender, EventArgs e)
        {
            if (!mIsEditMode) // We're going to edit-mode
            {
                // Show the input panel
                mSip.Enabled       = true;
                mVScrollBar.Height = this.Height - cTopOffset - 33; // 33 is the height of the Sip
                // Make sure the edit gui is created
                BuildEditGui();
            }
            else // We're leaving edit-mode
            {
                // Save changes
                SaveChangedResults();
                // Hide input panel
                mSip.Enabled       = false;
                mVScrollBar.Height = this.Height - cTopOffset;
            }

            mIsEditMode = !mIsEditMode;
            UpdateControls();
            Invalidate();
        }
        
        /// <summary>
        /// A charcater is entered in a goal text box. 
        /// Checks that it's a digit or backspace.
        /// </summary>
        private void TextBoxGoals_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (!(Char.IsDigit(e.KeyChar) || e.KeyChar == 8))
            {
                e.Handled = true;
            }
        }

        /// <summary>
        /// The parent control is changed. 
        /// </summary>
        protected override void OnParentChanged(EventArgs e)
        {
            base.OnParentChanged(e);
        }

        /// <summary>
        /// Draws the team names on the screen.
        /// </summary>
        protected override void OnPaint(PaintEventArgs e)
        {
            int   y = cTopOffset;
            Match match;
            int   matchesPerPage = (mIsEditMode) ? mMatchesPerPageEdit : mMatchesPerPage;

            // Load data if necessary
            if (mMatchList == null)
            {
                LoadData();
            }

            CreateOffscreenBitmap();

            for (int i = 0; i < Math.Min(matchesPerPage, mMatchList.Count); i++)
            {
                match = (Match)mMatchList[i + mScrollPos];
                mOffscreenGraphics.DrawString(match.HomeTeamName, cMainFont, cDataBrush, 52, y);
                mOffscreenGraphics.DrawString(match.AwayTeamName, cMainFont, cDataBrush, 122, y);
                if (!mIsEditMode)
                {
                    mOffscreenGraphics.DrawString(String.Format("{0}/{1}", match.Date.Day, match.Date.Month), cMainFont, cDataBrush, 8, y);
                    DrawStringRight(mOffscreenGraphics, match.HomeGoalsString + " -", cMainFont, cDataBrush, 217, y);
                    mOffscreenGraphics.DrawString(match.AwayGoalsString, cMainFont, cDataBrush, 221, y);
                }
                y += cRowHeight;
            }

            e.Graphics.DrawImage(mOffscreenBitmap, 0, 0);
        }

        /// <summary>
        /// Checks if the round label was tapped.
        /// </summary>
        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseUp(e);

            if (e.X > (this.ClientSize.Width / 2 - 30) && e.X < (this.ClientSize.Width / 2 + 30)
                && e.Y > 1 && e.Y < 17)
            {                
                // Show the "Select round:" dialog
                SelectRoundForm roundForm = new SelectRoundForm(mAppState.CurrentLeague.RoundCount);
                roundForm.ShowDialog();

                // Show the new round if on was selected
                if (roundForm.SelectedRound > -1)
                {
                    SaveChangedResults();
                    mRound = roundForm.SelectedRound;
                    InvalidateData();
                    Invalidate();
                }
            }
        }

        #endregion

        /// <summary>
        /// Method for building controls.
        /// </summary>
        public override void BuildGui()
        {
            if (!mIsGuiBuilt)
            {
                // We now know the height of this control so we use that to calculate
                // the number of matches that we can display before we need scrolling.
                mMatchesPerPage = (int)Math.Floor((this.Height - cTopOffset) / cRowHeight);
                // In edit-mode we need some extra room for the inputpanel
                mMatchesPerPageEdit = mMatchesPerPage - 2;

                // mSip
                mSip = new InputPanel();

                // mLabelRound
                mLabelRound        = new Label();
                mLabelRound.Parent = this;
                mLabelRound.Bounds = new Rectangle(90, 1, 60, 16);
                mLabelRound.Font   = new Font("Tahoma", 9F, FontStyle.Underline);

                // Font used by prev and next buttons
                // We want to use Bookdings to display arrow symbols but if it's
                // not available we use Tahoma and < and > chars instead.
                Font prevNextFont = new Font("Bookdings", 12F, FontStyle.Regular);
                bool isBookdingsAvailable = (prevNextFont.Name == "Bookdings");
                if (!isBookdingsAvailable)
                {
                    prevNextFont = new Font("Tahoma", 10F, FontStyle.Regular);
                }

                // mButtonRoundPrev
                mButtonRoundPrev         = new Button();
                mButtonRoundPrev.Parent  = this;
                mButtonRoundPrev.Bounds  = new Rectangle(66, 2, 15, 15);
                mButtonRoundPrev.Font    = prevNextFont;
                mButtonRoundPrev.Text    = (isBookdingsAvailable) ? "3" : "<";
                mButtonRoundPrev.Click  += new EventHandler(this.ButtonRoundPrev_Click);

                // mButtonRoundNext
                mButtonRoundNext         = new Button();
                mButtonRoundNext.Parent  = this;
                mButtonRoundNext.Bounds  = new Rectangle(150, 2, 15, 15);
                mButtonRoundNext.Font    = prevNextFont;
                mButtonRoundNext.Text    = (isBookdingsAvailable) ? "4" : ">";
                mButtonRoundNext.Click  += new EventHandler(this.ButtonRoundNext_Click);

                // mCheckBoxEdit
                mCheckBoxEdit        = new CheckBox();
                mCheckBoxEdit.Parent = this;
                mCheckBoxEdit.Text   = Language.GetString("matchpage", "edit");
                Graphics g = this.Parent.Parent.CreateGraphics();
                int width  = (int)g.MeasureString(mCheckBoxEdit.Text, mCheckBoxEdit.Font).Width + 24;
                mCheckBoxEdit.Bounds = new Rectangle(this.ClientSize.Width - width, 2, width, 15);
                g.Dispose();
                mCheckBoxEdit.Click  += new EventHandler(CheckBoxEdit_Click);

                // vScrollBar
                mVScrollBar               = new VScrollBar();
                mVScrollBar.Visible       = false;
                mVScrollBar.Parent        = this;
                mVScrollBar.Bounds        = new Rectangle(232, cTopOffset, 8, this.Height - cTopOffset);
                mVScrollBar.Minimum       = 0;
                mVScrollBar.ValueChanged += new EventHandler(this.VScrollBar_ValueChanged);

                mIsGuiBuilt = true;
                InitPage();
            }
        }

        /// <summary>
        /// Creates the controls that is used when the form is in edit mode.
        /// </summary>
        private void BuildEditGui()
        {
            if (!mIsEditGuiBuilt)
            {
                // DateTimePicker and goals
                mDateTimePickerMatchDate = new DateTimePicker[mMatchesPerPageEdit];
                mTextBoxHomeGoals        = new TextBox[mMatchesPerPageEdit];
                mTextBoxAwayGoals        = new TextBox[mMatchesPerPageEdit];
                KeyPressEventHandler keyPressEventHandler   = new KeyPressEventHandler(TextBoxGoals_KeyPress);
                EventHandler datePickerDropDowmEventHandler = new EventHandler(DateTimePickerMatchDate_DropDown);
                EventHandler datePickerCloseUpEventHandler  = new EventHandler(DateTimePickerMatchDate_CloseUp);

                // Loop over the number of matches displayd per page and initialize
                // date and goals controls.
                for (int i = 0; i < mMatchesPerPageEdit; i++)
                {
                    int y = cTopOffset + (i * cRowHeight);

                    // dateTimePickerMatchDate
                    mDateTimePickerMatchDate[i]           = new DateTimePicker();
                    mDateTimePickerMatchDate[i].Parent    = this;
                    mDateTimePickerMatchDate[i].Bounds    = new Rectangle(1, y, 49, 16);
                    mDateTimePickerMatchDate[i].DropDown += datePickerDropDowmEventHandler;
                    mDateTimePickerMatchDate[i].CloseUp  += datePickerCloseUpEventHandler;
                    mDateTimePickerMatchDate[i].Name      = i.ToString();

                    // textBoxHomeGoals
                    mTextBoxHomeGoals[i]           = new TextBox();
                    mTextBoxHomeGoals[i].Parent    = this;
                    mTextBoxHomeGoals[i].Bounds    = new Rectangle(192, y, 20, 16);
                    mTextBoxHomeGoals[i].MaxLength = 2;
                    mTextBoxHomeGoals[i].KeyPress += keyPressEventHandler;

                    // textBoxAwayGoals
                    mTextBoxAwayGoals[i]           = new TextBox();
                    mTextBoxAwayGoals[i].Parent    = this;
                    mTextBoxAwayGoals[i].Bounds    = new Rectangle(212, y, 20, 16);
                    mTextBoxAwayGoals[i].MaxLength = 2;
                    mTextBoxAwayGoals[i].KeyPress += keyPressEventHandler;

                }

                mIsEditGuiBuilt = true;
            }
        }

        /// <summary>
        /// Invalidates the data for this page.
        /// </summary>
        public void InvalidateData()
        {
            mMatchList = null;
        }

        /// <summary>
        /// Method for initializing controls when 
        /// league (or league data) changes.
        /// </summary>
        public override void InitPage() 
        {
            // Only do this if components are initialized
            if (mIsGuiBuilt) 
            {
                mIsEditMode = false;
                mCheckBoxEdit.Checked = false;
                
                mRound = mAppState.CurrentLeague.GetLastPlayedRound();
            }

            InvalidateData();
            // This call is necessary due to what seems to 
            // be a bug with redrawing tabpages.
            Invalidate();
        }

        /// <summary>
        /// Loads the matches for the current round.
        /// </summary>
        private void LoadData()
        {
            mMatchList = mAppState.CurrentLeague.GetMatchesForRound(mRound);

            UpdateControls();
        }

        /// <summary>
        /// Updates the controls when the round changes.
        /// </summary>
        private void UpdateControls()
        {
            Match match;
            bool  visible;
            int   matchesPerPage = (mIsEditMode) ? mMatchesPerPageEdit : mMatchesPerPage;

            // Sanity check of scrollbar position
            if (mScrollPos > (mMatchList.Count-matchesPerPage))
            {
                // If mVScrollBar.Value is actually changed mScrollPos
                // is also changed in the ValueChanged event handler
                mVScrollBar.Value = 0;
            }

            if (mIsEditGuiBuilt)
            {
                // Loop through matches and update the date and goals controls
                for (int i = 0; i < mMatchesPerPageEdit; i++)
                {
                    visible = (mIsEditMode && (i < mMatchList.Count));

                    mDateTimePickerMatchDate[i].Visible = visible;
                    mTextBoxHomeGoals[i].Visible        = visible;
                    mTextBoxAwayGoals[i].Visible        = visible;

                    if (visible)
                    {
                        match = (Match)mMatchList[i + mScrollPos];
                        mDateTimePickerMatchDate[i].Value = match.Date;
                        mTextBoxHomeGoals[i].Text = match.HomeGoalsString;
                        mTextBoxAwayGoals[i].Text = match.AwayGoalsString;
                    }
                }
            }

            // Show scrollbar if necessary
            mVScrollBar.Visible = (mMatchList.Count > matchesPerPage);
            if (mVScrollBar.Visible)
            {
                mVScrollBar.LargeChange = matchesPerPage;
                mVScrollBar.Maximum = 
                    Math.Max(0, mMatchList.Count-matchesPerPage) + (matchesPerPage - 1);
            }

            // Update round label and buttons
            mLabelRound.Text  = Language.GetString("matchpage", "round") + " " + mRound;
            Graphics g        = this.Parent.Parent.CreateGraphics();
            mLabelRound.Width = (int)g.MeasureString(mLabelRound.Text, mLabelRound.Font).Width + 2;
            g.Dispose();
            mLabelRound.Left      = (this.ClientSize.Width - mLabelRound.Width) / 2;
            mButtonRoundPrev.Left = mLabelRound.Left - mButtonRoundPrev.Width - 6;
            mButtonRoundNext.Left = mLabelRound.Right + 6;
        }

        /// <summary>
        /// Saves any changed results. Returns true if any changes were
        /// actually saved, false otherwise.
        /// </summary>
        public void SaveChangedResults()
        {           
            if (mIsEditMode) 
            {
                Match match;

                // Loop through all the match controls
                for (int i = 0; i < Math.Min(mMatchesPerPageEdit, mMatchList.Count); i++)
                {
                    match = (Match)mMatchList[i + mScrollPos];

                    // Only save if the result has been changed
                    if (mTextBoxHomeGoals[i].Modified || mTextBoxAwayGoals[i].Modified)
                    {
                        try 
                        {
                            int homeGoals = Int32.Parse(mTextBoxHomeGoals[i].Text);
                            int awayGoals = Int32.Parse(mTextBoxAwayGoals[i].Text);
                            match.SetResult(homeGoals, awayGoals);
                        }
                        catch (FormatException)
                        {
                            match.SetNotPlayed();
                        }

                        match.Save();
                    }
                }
            }
        }

    }
}
