/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Drawing;
using System.Windows.Forms;
using MBovin.TeamStats;
using MBovin.TeamStats.Core;
using MBovin.TeamStats.Util;

namespace MBovin.TeamStats.UI
{
    #region Enums

    /// <summary>
    /// Enum for match mode.
    /// </summary>
    public enum TableMatchesMode
    {
        All,
        Home,
        Away
    }
    
    /// <summary>
    /// Enum for round mode.
    /// </summary>
    public enum TableRoundsMode
    {
        All,
        First,
        Last
    }

    /// <summary>
    /// Enum for table view mode.
    /// </summary>
    public enum TableViewMode
    {
        Goals,
        Result
    }
    
    #endregion

    /// <summary>
	/// This class impelements the table page.
	/// </summary>
	public class TablePage : PageBase
	{
        // Constants
        private const int cTopOffset = 22;
        private const int cRowHeight = 13;

        // Controls
        private ComboBox      mComboBoxMatches;
        private ComboBox      mComboBoxRounds;
        private NumericUpDown mNumericUpDownRounds;
        private Button        mButtonTableView;
        private VScrollBar    mVScrollBar;
        
        // Page settings
        private TableViewMode mTableViewMode;
        private int           mTeamsPerPage;

        // The data for this page
        private Table mTable;


        /// <summary>
        /// Constructor
        /// </summary>
        public TablePage(AppState appState) : base(appState) 
        {
            this.Text = Language.GetString("tablepage", "title");
        }

        #region Events

        /// <summary>
        /// The matches setting is changed.
        /// </summary>
        private void ComboBoxMatches_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            InvalidateData();
            Invalidate();        
        }

        /// <summary>
        /// The round setting is changed.
        /// </summary>
        private void ComboBoxRounds_SelectedIndexChanged(object sender, System.EventArgs e)
        {
            mNumericUpDownRounds.Visible = (mComboBoxRounds.SelectedIndex != 0);
            mButtonTableView.Left = (mNumericUpDownRounds.Visible) ? 206 : 154;
            InvalidateData();
            Invalidate();        
        }

        /// <summary>
        /// The number of matches is changed.
        /// </summary>
        private void NumericUpDownRounds_ValueChanged(object sender, System.EventArgs e)
        {
            InvalidateData();
            Invalidate();
        }

        /// <summary>
        /// The scrollbar is used.
        /// </summary>
        private void VScrollBar_ValueChanged(object sender, System.EventArgs e)
        {
            Invalidate();
        }

        /// <summary>
        /// The table view is changed.
        /// </summary>
        private void ButtonTableView_Click(object sender, System.EventArgs e)
        {
            mTableViewMode = (TableViewMode)(((int)mTableViewMode + 1) % 2);
            Invalidate();
        }

        /// <summary>
        /// Draws the table on the screen.
        /// </summary>
        protected override void OnPaint(PaintEventArgs e)
        {
            // Get settings from controls
            TableMatchesMode matchesMode = (TableMatchesMode)mComboBoxMatches.SelectedIndex;
            TableRoundsMode  roundsMode  = (TableRoundsMode)mComboBoxRounds.SelectedIndex;
            bool includeHome = ((matchesMode == TableMatchesMode.All) || 
                                (matchesMode == TableMatchesMode.Home));
            bool includeAway = ((matchesMode == TableMatchesMode.All) || 
                                (matchesMode == TableMatchesMode.Away));
            bool isForward = (roundsMode != TableRoundsMode.Last);
            int maxMatches = 
                (roundsMode == TableRoundsMode.All) ? -1 : (int)mNumericUpDownRounds.Value;

            // Create the league and table objects
            League league = mAppState.CurrentLeague;
            if (mTable == null)
            {
                mTable = new Table(league, includeHome, includeAway, isForward, maxMatches);
            }

            CreateOffscreenBitmap();
            
            // Start pos from top 
            int y = cTopOffset;
            // How many teams to draw
            int stopAt = Math.Min(mTeamsPerPage, league.TeamCount) + mVScrollBar.Value;
            TableTeam tableTeam;

            // Loop through the teams
            for (int i = mVScrollBar.Value; i < stopAt; i++)
            {
                tableTeam = (TableTeam)mTable.TeamList[i];
                DrawStringRight(mOffscreenGraphics, (i+1).ToString() + ".", cMainFont, cDataBrush, 20, y);
                mOffscreenGraphics.DrawString(tableTeam.Name, cMainFont, cDataBrush, 21, y);
                DrawStringRight(mOffscreenGraphics, tableTeam.Played.ToString(), cMainFont, cDataBrush, 116, y);
                if (mTableViewMode == TableViewMode.Goals)
                {    
                    DrawStringRight(mOffscreenGraphics, tableTeam.For.ToString(), cMainFont, cDataBrush, 142, y);
                    mOffscreenGraphics.DrawString("- " + tableTeam.Against.ToString(), cMainFont, cDataBrush, 145, y);
                    DrawStringRight(mOffscreenGraphics, tableTeam.Points.ToString(), cMainFont, cDataBrush, 191, y);
                    DrawStringRight(mOffscreenGraphics, tableTeam.GoalDiff.ToString("+0;-0;+0"), cMainFont, cDataBrush, 222, y);
                }
                else // TableViewMode.Results
                {
                    DrawStringRight(mOffscreenGraphics, tableTeam.Won.ToString(), cMainFont, cDataBrush, 144, y);
                    DrawStringRight(mOffscreenGraphics, tableTeam.Drawn.ToString(), cMainFont, cDataBrush, 169, y);
                    DrawStringRight(mOffscreenGraphics, tableTeam.Lost.ToString(), cMainFont, cDataBrush, 194, y);
                    DrawStringRight(mOffscreenGraphics, tableTeam.Points.ToString(), cMainFont, cDataBrush, 222, y);
                }
                
                // Draw table lines
                if (league.IsTableLine(i+1))
                {
                    mOffscreenGraphics.DrawLine(cLabelPen, 2, y + (cRowHeight+1), 222, y + (cRowHeight+1));
                }

                y += cRowHeight;
            }

            e.Graphics.DrawImage(mOffscreenBitmap, 0, 0);
        }

        #endregion 

        /// <summary>
        /// Method for building controls.
        /// </summary>
        public override void BuildGui()
        {
            if (!mIsGuiBuilt)
            {
                // We now know the height of this control so we use that to calculate
                // the number of teams that we can display before we need scrolling.
                mTeamsPerPage = (int)Math.Floor((this.Height - cTopOffset) / cRowHeight);

                // comboBoxMatches
                mComboBoxMatches        = new ComboBox();
                mComboBoxMatches.Parent = this;
                mComboBoxMatches.Bounds = new Rectangle(2, 1, 72, 20);
                mComboBoxMatches.Items.Add(Language.GetString("tablepage", "matchesall"));
                mComboBoxMatches.Items.Add(Language.GetString("tablepage", "matcheshome"));
                mComboBoxMatches.Items.Add(Language.GetString("tablepage", "matchesaway"));
                mComboBoxMatches.SelectedIndexChanged += new EventHandler(this.ComboBoxMatches_SelectedIndexChanged);

                // comboBoxRounds
                mComboBoxRounds        = new ComboBox();
                mComboBoxRounds.Parent = this;
                mComboBoxRounds.Bounds = new Rectangle(78, 1, 72, 20);
                mComboBoxRounds.Items.Add(Language.GetString("tablepage", "roundsall"));
                mComboBoxRounds.Items.Add(Language.GetString("tablepage", "roundsfirst"));
                mComboBoxRounds.Items.Add(Language.GetString("tablepage", "roundslast"));
                mComboBoxRounds.SelectedIndexChanged += new EventHandler(this.ComboBoxRounds_SelectedIndexChanged);
            
                // numericUpDownRounds
                mNumericUpDownRounds         = new NumericUpDown();
                mNumericUpDownRounds.Parent  = this;
                mNumericUpDownRounds.Bounds  = new Rectangle(154, 1, 48, 20);
                mNumericUpDownRounds.Maximum = new System.Decimal(new int[] { 99, 0, 0, 0});
                mNumericUpDownRounds.Minimum = new System.Decimal(new int[] { 1, 0, 0, 0});
                mNumericUpDownRounds.Value   = new System.Decimal(new int[] { 5, 0, 0, 0});
                mNumericUpDownRounds.ValueChanged += new EventHandler(this.NumericUpDownRounds_ValueChanged);

                // buttonTableView
                mButtonTableView        = new Button();
                mButtonTableView.Parent = this;
                mButtonTableView.Bounds = new Rectangle(154, 1, 18, 20);
                mButtonTableView.Text   = "=";
                mButtonTableView.Click += new EventHandler(this.ButtonTableView_Click);

                // vScrollBar
                mVScrollBar               = new VScrollBar();
                mVScrollBar.Parent        = this;
                mVScrollBar.Bounds        = new Rectangle(226, 23, 14, this.Height - 23);
                mVScrollBar.Minimum       = 0;
                mVScrollBar.LargeChange   = mTeamsPerPage;
                mVScrollBar.ValueChanged += new EventHandler(this.VScrollBar_ValueChanged);

                mIsGuiBuilt = true;
            }
        }

        /// <summary>
        /// Invalidates the data for this page.
        /// </summary>
        public void InvalidateData()
        {
            mTable = null;
        }

        /// <summary>
        /// Method for initializing controls when 
        /// league (or league data) changes.
        /// </summary>
        public override void InitPage() 
        {
            // Only do this if components are initialized
            if (mIsGuiBuilt)
            {
                League league = mAppState.CurrentLeague;

                mTableViewMode = TableViewMode.Goals;
                mComboBoxMatches.SelectedIndex = 0;
                mComboBoxRounds.SelectedIndex  = 0;
                mNumericUpDownRounds.Value     = 5;
                mNumericUpDownRounds.Visible   = false;
                mVScrollBar.Maximum = 
                    Math.Max(0, league.TeamCount-mTeamsPerPage) + (mTeamsPerPage - 1);
                mVScrollBar.Value = 0;
            }
            InvalidateData();
        }

	}
}
