/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;
using System.Drawing;
using System.Windows.Forms;
using MBovin.TeamStats;
using MBovin.TeamStats.Core;
using MBovin.TeamStats.Util;

namespace MBovin.TeamStats.UI
{
    
    #region Enums

    /// <summary>
    /// Enum for match mode.
    /// </summary>
    public enum TeamMatchesMode
    {
        All,
        Home,
        Away
    }
    
    /// <summary>
    /// Enum for team view mode.
    /// </summary>
    public enum TeamViewMode
    {
        Table,
        Matches
    }
    
    #endregion
    
    /// <summary>
	/// This class implements the team page.
	/// </summary>
	public class TeamPage : PageBase
	{
        // Constants
        private const int cTableViewHeight = 200;
        private const int cScrollFactor    = 20;
        private const int cTopOffset       = 22;
        private const int cRowHeight       = 14;

        // Controls
        private ComboBox     mComboBoxTeams;
        private ComboBox     mComboBoxMatches;
        private Button       mButtonView;
        private VScrollBar   mVScrollBar;
        private EventHandler mVScrollbar_ValueChanged;
        
        // Page settings
        private TeamViewMode mViewMode;
        private int          mMatchesPerPage;
        private int          mPageHeight;
        
        // The data for this page
        private Table     mTable;
        private ArrayList mMatchList;


        /// <summary>
        /// Contructor.
        /// </summary>
        public TeamPage(AppState appState) : base(appState)
		{
            this.Text   = Language.GetString("teampage", "title");
        }

        #region Events

        /// <summary>
        /// A new team is selected.
        /// </summary>
        private void ComboBoxTeams_SelectedIndexChanged(object sender, EventArgs e)
        {
            InvalidateData();
            Invalidate();
        }

        /// <summary>
        /// All, Home or Away is changed.
        /// </summary>
        private void ComboBoxMatches_SelectedIndexChanged(object sender, EventArgs e)
        {
            InvalidateData();
            Invalidate();
        }

        /// <summary>
        /// The view button is clicked.
        /// </summary>
        private void ButtonView_Click(object sender, EventArgs e)
        {
            mViewMode = (mViewMode == TeamViewMode.Matches) ? 
                TeamViewMode.Table : 
                TeamViewMode.Matches;
            mButtonView.Text = (mViewMode == TeamViewMode.Matches) ? 
                Language.GetString("teampage", "viewmatches") :
                Language.GetString("teampage", "viewstats");

            SetupScrollBar();
            Invalidate();
        }

        /// <summary>
        /// The scrollbar is used.
        /// </summary>
        private void VScrollBar_ValueChanged(object sender, EventArgs e)
        {
            Invalidate();
        }

        /// <summary>
        /// Draws the data on the screen.
        /// </summary>
        protected override void OnPaint(PaintEventArgs e)
        {
            // Load new data if necessary
            if (mTable == null)
            {
                LoadData();
                SetupScrollBar();
            }

            CreateOffscreenBitmap(mPageHeight);

            int yOffset = 0;
            switch (mViewMode)
            {
                case TeamViewMode.Table : 
                    DrawTable();
                    yOffset = -(mVScrollBar.Value * cScrollFactor);
                    break;

                case TeamViewMode.Matches :
                    DrawMatches();
                    break;
            }

            e.Graphics.DrawImage(mOffscreenBitmap, 0, yOffset);
        }

        #endregion

        /// <summary>
        /// Method for building controls.
        /// </summary>
        public override void BuildGui()
        {
            if (!mIsGuiBuilt)
            {
                mPageHeight     = Math.Max(this.Height, cTableViewHeight);
                mMatchesPerPage = (int)Math.Floor((this.Height - cTopOffset) / cRowHeight);

                // comboBoxTeams
                mComboBoxTeams        = new ComboBox();
                mComboBoxTeams.Parent = this;
                mComboBoxTeams.Bounds = new Rectangle(2, 1, 90, 20);
                mComboBoxTeams.SelectedIndexChanged += new EventHandler(ComboBoxTeams_SelectedIndexChanged);

                // comboBoxMatches
                mComboBoxMatches        = new ComboBox();
                mComboBoxMatches.Parent = this;
                mComboBoxMatches.Bounds = new Rectangle(96, 1, 66, 20);
                mComboBoxMatches.Items.Add(Language.GetString("teampage", "matchesall"));
                mComboBoxMatches.Items.Add(Language.GetString("teampage", "matcheshome"));
                mComboBoxMatches.Items.Add(Language.GetString("teampage", "matchesaway"));
                mComboBoxMatches.SelectedIndexChanged += new EventHandler(ComboBoxMatches_SelectedIndexChanged);

                // buttonView
                mButtonView        = new Button();
                mButtonView.Parent = this;
                mButtonView.Bounds = new Rectangle(166, 1, 72, 20);
                mButtonView.Font   = MainForm.cButtonFont;
                mButtonView.Text   = Language.GetString("teampage", "viewstats");
                mButtonView.Click += new EventHandler(ButtonView_Click);

                // vScrollBar
                mVScrollBar               = new VScrollBar();
                mVScrollBar.Visible       = false;
                mVScrollBar.Parent        = this;
                mVScrollBar.Bounds        = new Rectangle(226, 23, 14, this.Height - 23);
                mVScrollBar.Minimum       = 0;
                mVScrollbar_ValueChanged  = new EventHandler(VScrollBar_ValueChanged);
                mVScrollBar.ValueChanged += mVScrollbar_ValueChanged;

                mIsGuiBuilt = true;
                InitPage();
            }
        }

        /// <summary>
        /// Invalidates the data on this page.
        /// </summary>
        public void InvalidateData()
        {
            mTable     = null;
            mMatchList = null;
        }

        /// <summary>
        /// Method for initializing controls when 
        /// league (or league data) changes.
        /// </summary>
        public override void InitPage()
        {
            // Only do this if components are initialized
            if (mIsGuiBuilt)
            {
                // Remember previously selected team - if any
                string previousTeamName = (mComboBoxTeams.SelectedItem != null) ?
                    ((Team)mComboBoxTeams.SelectedItem).Name : null;

                // Add teams to team combobox
                mComboBoxTeams.Items.Clear();
                League league = mAppState.CurrentLeague;
                ArrayList teamList = league.GetTeams();
                teamList.Sort();           
                int selectedIndex = 0;
                int i = 0;
                foreach (Team team in teamList)
                {
                    mComboBoxTeams.Items.Add(team);
                    // Re-select the team if the same as the previously selected
                    if (team.Name.Equals(previousTeamName))
                    {
                        selectedIndex = i;
                    }
                    i++;
                }
                
                // Reset controls
                mComboBoxTeams.SelectedIndex   = selectedIndex;
                mComboBoxMatches.SelectedIndex = 0;
                mViewMode                      = TeamViewMode.Table;
                mButtonView.Text               = Language.GetString("teampage", "viewstats");
            }

            InvalidateData();
        }

        /// <summary>
        /// Configures the scrollbar depending on the view mode.
        /// </summary>
        private void SetupScrollBar()
        {
            if (mViewMode == TeamViewMode.Table)
            {
                mVScrollBar.Maximum     = (mPageHeight - 1) / cScrollFactor;
                mVScrollBar.LargeChange = this.Height / cScrollFactor;

                // We need to disable the event handler when we update the scroll bar
                mVScrollBar.ValueChanged -= mVScrollbar_ValueChanged;
                mVScrollBar.Value = 0;
                mVScrollBar.ValueChanged += mVScrollbar_ValueChanged;

                mVScrollBar.Visible = (mPageHeight > this.Height);
            }
            else // TeamViewMode.Matches
            {
                mVScrollBar.Maximum     = Math.Max(0, mMatchList.Count - mMatchesPerPage) + (mMatchesPerPage - 1);
                mVScrollBar.LargeChange = mMatchesPerPage;
                // Set the scrollbar so that the first not-yet-played match is centered on the page
                int scrollValue = GetCurrentMatchIndex() - (3 * (mMatchesPerPage / 4));
                // Sanity check of scroll value
                if (scrollValue < 0) scrollValue = 0;
                if (scrollValue > mVScrollBar.Maximum) scrollValue = mVScrollBar.Maximum;

                // We need to disable the event handler when we update the scroll bar
                mVScrollBar.ValueChanged -= mVScrollbar_ValueChanged;
                mVScrollBar.Value = scrollValue;
                mVScrollBar.ValueChanged += mVScrollbar_ValueChanged;

                mVScrollBar.Visible = true;
            }
        }

        /// <summary>
        /// Loads the data for this page.
        /// </summary>
        private void LoadData()
        {
            // Get settings from controls
            TeamMatchesMode matchesMode = (TeamMatchesMode)mComboBoxMatches.SelectedIndex;
            bool includeHome = 
                ((matchesMode == TeamMatchesMode.All) || (matchesMode == TeamMatchesMode.Home));
            bool includeAway = 
                ((matchesMode == TeamMatchesMode.All) || (matchesMode == TeamMatchesMode.Away));
            Team selectedTeam = (Team)mComboBoxTeams.SelectedItem;

            // Get the table and match data
            League league = mAppState.CurrentLeague;
            mTable        = new Table(league, includeHome, includeAway, true, -1);
            mMatchList    = league.GetMatchesForTeam(selectedTeam.Index, includeHome, includeAway);
        }

        /// <summary>
        /// Returns a list of the most frequent results for the specified team.
        /// </summary>
        private ArrayList GetFrequentResult(Team team)
        {
            ArrayList resultList = new ArrayList(10);

            foreach (Match match in mMatchList)
            {
                if (match.IsPlayed)
                {
                    AddResult(resultList, match.GetGoalsFor(team), match.GetGoalsAgainst(team));
                }
            }

            resultList.Sort();
            return resultList;
        }

        /// <summary>
        /// Adds a result to the specified list of frequent result.
        /// </summary>
        private void AddResult(ArrayList resultList, int goalsFor, int goalsAgainst)
        {
            bool resultFound = false;
            
            // Loop through all the results in the list
            foreach (Result result in resultList)
            {
                // If the result is found -> bump the count for
                // that result and exit loop
                if (result.IsResult(goalsFor, goalsAgainst))
                {
                    result.Count++;
                    resultFound = true;
                    break;
                }
            }

            // This was a new result -> add it to the list
            if (!resultFound)
            {
                resultList.Add(new Result(goalsFor, goalsAgainst));
            }
        }

        /// <summary>
        /// Returns the index of the first not-yet-played match in the 
        /// current match list.
        /// </summary>
        private int GetCurrentMatchIndex()
        {
            int i;
            for (i = 0; i < mMatchList.Count; i++)
            {
                if (!((Match)mMatchList[i]).IsPlayed) 
                {
                    break;
                }
            }
            return i;
        }

        /// <summary>
        /// Draws the table view.
        /// </summary>
        private void DrawTable()
        {
            Team      team = (Team)mComboBoxTeams.SelectedItem;
            TableTeam tableTeam = mTable.GetTableTeam(team);

            int x1 = Language.GetNumber("teampage", "x1stats");
            int x2 = Language.GetNumber("teampage", "x2stats");
            int x3 = Language.GetNumber("teampage", "x3stats");
            int x4 = Language.GetNumber("teampage", "x4stats");
            int y  = 25;
            int ydelta1 = 16;
            int ydelta2 = 19;

            mOffscreenGraphics.DrawString(Language.GetString("teampage", "tableposition"), cMainFont, cLabelBrush, x1, y);
            DrawStringRight(mOffscreenGraphics, tableTeam.Position.ToString(), cMainFont, cDataBrush, x2, y);
            y += ydelta2;

            mOffscreenGraphics.DrawString(Language.GetString("teampage", "matchesplayed"), cMainFont, cLabelBrush, x1, y);
            DrawStringRight(mOffscreenGraphics, tableTeam.Played.ToString(), cMainFont, cDataBrush, x2, y);
            y += ydelta1;
            mOffscreenGraphics.DrawString(Language.GetString("teampage", "wins"), cMainFont, cLabelBrush, x1, y);
            DrawStringRight(mOffscreenGraphics, tableTeam.Won.ToString(), cMainFont, cDataBrush, x2, y);
            DrawStringRight(mOffscreenGraphics, "(" + tableTeam.WonPercent.ToString("##0.0%") + ")", cMainFont, cDataBrush, x3, y);
            y += ydelta1;
            mOffscreenGraphics.DrawString(Language.GetString("teampage", "draws"), cMainFont, cLabelBrush, x1, y);
            DrawStringRight(mOffscreenGraphics, tableTeam.Drawn.ToString(), cMainFont, cDataBrush, x2, y);
            DrawStringRight(mOffscreenGraphics, "(" + tableTeam.DrawnPercent.ToString("##0.0%") + ")", cMainFont, cDataBrush, x3, y);
            y += ydelta1;
            mOffscreenGraphics.DrawString(Language.GetString("teampage", "losses"), cMainFont, cLabelBrush, x1, y);
            DrawStringRight(mOffscreenGraphics, tableTeam.Lost.ToString(), cMainFont, cDataBrush, x2, y);
            DrawStringRight(mOffscreenGraphics, "(" + tableTeam.LostPercent.ToString("##0.0%") + ")", cMainFont, cDataBrush, x3, y);
            y += ydelta2;

            mOffscreenGraphics.DrawString(Language.GetString("teampage", "goalsfor"), cMainFont, cLabelBrush, x1, y);
            DrawStringRight(mOffscreenGraphics, tableTeam.For.ToString(), cMainFont, cDataBrush, x2, y);
            DrawStringRight(mOffscreenGraphics, "(" + tableTeam.ForAvg.ToString("#0.00") + ")", cMainFont, cDataBrush, x3, y);
            y += ydelta1;
            mOffscreenGraphics.DrawString(Language.GetString("teampage", "goalsagainst"), cMainFont, cLabelBrush, x1, y);
            DrawStringRight(mOffscreenGraphics, tableTeam.Against.ToString(), cMainFont, cDataBrush, x2, y);
            DrawStringRight(mOffscreenGraphics, "(" + tableTeam.AgainstAvg.ToString("#0.00") + ")", cMainFont, cDataBrush, x3, y);
            y += ydelta2;

            mOffscreenGraphics.DrawString(Language.GetString("teampage", "frequentresult"), cMainFont, cLabelBrush, x1, y);

            ArrayList resultList = GetFrequentResult(team);
            Result    result;
            string    s;

            for (int i = 0; i < 3; i++)
            {
                if (i < resultList.Count)
                {
                    result = resultList[i] as Result;
                    s = String.Format("{0}-{1}  {2} {3}", 
                        result.HomeGoals, 
                        result.AwayGoals, 
                        result.Count, 
                        Language.GetString("teampage", "times"));
                    mOffscreenGraphics.DrawString(s, cMainFont, cDataBrush, x4, y);
                    y += ydelta1;
                }
            }

        }

        /// <summary>
        /// Draws the match view.
        /// </summary>
        private void DrawMatches()
        {
            int x1 = 28;
            int x2 = 55;
            int x3 = 143;
            int x4 = 168;
            int x5 = 202;
            int y  = cTopOffset;

            Team   team = (Team)mComboBoxTeams.SelectedItem;
            Match  match;
            int    goalsFor, goalsAgainst;
            string placeLetter, resultLetter;

            // How many matches to draw
            int stopAt = Math.Min(mMatchesPerPage, mMatchList.Count) + mVScrollBar.Value;

            for (int i = mVScrollBar.Value; i < stopAt; i++)
            {
                match        = mMatchList[i] as Match;
                goalsFor     = match.GetGoalsFor(team);
                goalsAgainst = match.GetGoalsAgainst(team);
                placeLetter  = (match.HomeTeamId == team.Index) ? 
                    "(" + Language.GetString("teampage", "homeletter") + ")" :
                    "(" + Language.GetString("teampage", "awayletter") + ")";
                if      (goalsFor > goalsAgainst) resultLetter = Language.GetString("teampage", "winletter");
                else if (goalsFor < goalsAgainst) resultLetter = Language.GetString("teampage", "lossletter");
                else                              resultLetter = Language.GetString("teampage", "drawletter");

                DrawStringRight(mOffscreenGraphics, match.Date.Day + "/", cMainFont, cDataBrush, x1, y);
                mOffscreenGraphics.DrawString(match.Date.Month.ToString(), cMainFont, cDataBrush, x1 + 1, y);
                mOffscreenGraphics.DrawString(match.GetOpponent(team).Name, cMainFont, cDataBrush, x2, y);
                mOffscreenGraphics.DrawString(placeLetter, cMainFont, cDataBrush, x3, y);
                if (match.IsPlayed)
                {                    
                    mOffscreenGraphics.DrawString(goalsFor + "-" + goalsAgainst, cMainFont, cDataBrush, x4, y);
                    DrawStringCenter(mOffscreenGraphics, resultLetter, cMainFont, cDataBrush, x5, y);
                }

                y += cRowHeight;
            }
        }

    }
}
