/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.3.0 February 2007
 * Copyright (c) 2004-07 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Drawing;
using System.IO;
using System.Net;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Xml;

namespace MBovin.TeamStats.Util
{
	/// <summary>
	/// This class contains various static helper methods.
	/// </summary>
    public class Utils
    {
        
        /// <summary>
        /// The directory name of the application (where the .exe-file is).
        /// </summary>
        public static string AppDirectory
        {
            get
            {
                return Path.GetDirectoryName(
                    System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase);
            }
        }

        /// <summary>
        /// Returns an Utf-8 encoded byte array of s.
        /// If s is longer than 'length' it is truncated.
        /// The size of the byte array is always length * 4
        /// </summary>
        public static byte[] GetUtf8Bytes(string s, int length)
        {
            if (s.Length > length)
            {
                s = s.Substring(0, length);
            }
            byte[] tmpBytes = new UTF8Encoding().GetBytes(s);
            // Create a new byte array of the correct size and copy into it
            byte[] bytes    = new byte[length * 4];
            tmpBytes.CopyTo(bytes, 0);

            return bytes;
        }

        /// <summary>
        /// Parses s using the Convert.ToInt32()-method. If that triggers 
        /// a FormatException, the defaultValue is returned.
        /// </summary>
        public static int StringToInt32(string s, int defaultValue)
        {
            try 
            {
                return Convert.ToInt32(s);
            }
            catch (FormatException) 
            {
                return defaultValue;
            }
        }

        /// <summary>
        /// Loads an icon that has been stored as an 
        /// embedded resource in the assembly.
        /// </summary>
        public static Icon LoadEmbeddedIcon(string name, int width, int height)
        {
            Icon   icon   = null;
            Stream stream = null;
            try
            {
                stream = Assembly.GetExecutingAssembly().GetManifestResourceStream(name);
                icon   = new Icon(stream, width, height);
            }
            finally
            {
                if (stream != null) stream.Close();
            }

            return icon;
        }

        /// <summary>
        /// Loads and returns an xml-document from the specified url.
        /// </summary>
        public static XmlDocument DownloadXmlDocument(string url)
        {
            HttpWebRequest  request  = null;
            HttpWebResponse response = null;
            try
            {
                // Create the request object
                request = (HttpWebRequest)WebRequest.Create(url);
                request.Timeout = 30000;

                // Get a response and read the xml data from the response stream
                response = (HttpWebResponse)request.GetResponse();
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.Load(response.GetResponseStream());

                return xmlDoc;
            }
            finally
            {
                if (response != null) response.Close();
            }
        }

        /// <summary>
        /// Returns a unique device id.
        /// </summary>
        public static string GetDeviceID()
        {
            try
            {
                int    bufferLength = 256;	
                byte[] buffer       = new byte[bufferLength];
                int    cb           = 0;

                buffer[0] = 0;	
                buffer[1] = 1;	

                KernelIoControl(IOCTL_HAL_GET_DEVICEID, IntPtr.Zero, 0, buffer, bufferLength, ref cb);
			
                Int32 dwPresetIDOffsset  = BitConverter.ToInt32(buffer, 4);	
                Int32 dwPlatformIDOffset = BitConverter.ToInt32(buffer, 0xc);	
                Int32 dwPlatformIDSize   = BitConverter.ToInt32(buffer, 0x10);

                StringBuilder sb = new StringBuilder();	
                sb.Append(String.Format("{0:X8}-{1:X4}-{2:X4}-{3:X4}-",	
                    BitConverter.ToInt32(buffer, dwPresetIDOffsset),
                    BitConverter.ToInt16(buffer, dwPresetIDOffsset+4),
                    BitConverter.ToInt16(buffer, dwPresetIDOffsset+6),
                    BitConverter.ToInt16(buffer, dwPresetIDOffsset+8)));	
			
                for(int i = dwPlatformIDOffset; i < dwPlatformIDOffset + dwPlatformIDSize; i ++)
                {
                    sb.Append(String.Format("{0:X2}", buffer[i]));
                }

                return sb.ToString();
            }
            catch (Exception)
            {
                return "";
            }
        }

        #region External references

        // This stuff is used by GetDeviceID()
        private static Int32 FILE_DEVICE_HAL = 0x00000101;
        private static Int32 FILE_ANY_ACCESS = 0x0;
        private static Int32 METHOD_BUFFERED = 0x0;

        private static Int32 IOCTL_HAL_GET_DEVICEID = 
            ((FILE_DEVICE_HAL) << 16) | ((FILE_ANY_ACCESS) << 14) 
            | ((21) << 2) | (METHOD_BUFFERED);

        [DllImport("coredll.dll", SetLastError=true)]
        private static extern bool KernelIoControl(Int32 dwIoControlCode, IntPtr
            lpInBuf, Int32 nInBufSize, byte[] lpOutBuf, Int32 nOutBufSize, ref Int32
            lpBytesReturned);

        #endregion

	}
}
