/*
 * TSCreate by Mats Bovin (tsppc@mbovin.com)
 * v1.0.0 July 2004
 * Copyright (c) 2004 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;
using System.Collections.Specialized;
using System.IO;
using MBovin.TeamStats.Core;

namespace MBovin.TSCreate
{
    /// <summary>
    /// This class creates a TeamStats file from a semi-colon separated
    /// file with matches and some other parameteres.
    /// </summary>
    public class TSFile
    {
        public const int cLeagueNameLength = 16;
        public const int cTeamNameLength   = 10;

        private League    mLeague;
        private ArrayList mArrTeams;
        private ArrayList mArrMatches;
        private string    mWarningMessage = "";


        /// <summary>
        /// Constructor.
        /// </summary>
        public TSFile() {}

        /// <summary>
        /// Contains warning messages that occurred during loading of the file.
        /// </summary>
        public string[] Warnings
        {
            get 
            { 
                return (mWarningMessage == "") ? 
                    new string[0] : 
                    mWarningMessage.Split('|'); 
            }
        }

        /// <summary>
        /// Helper method that converts a string to an int.
        /// </summary>
        private int StringToInt(string s, int defaultValue)
        {
            try
            {
                return int.Parse(s);
            }
            catch
            {
                return defaultValue;
            }
        }

        /// <summary>
        /// Validates the params and throws ArgumentException if something is wrong.
        /// </summary>
        private void ValidateParams(NameValueCollection props)
        {
            if ((props["matchfile"] == null) || (props["matchfile"].Trim().Length == 0))
            {
                throw new ArgumentException("You must specify a match file.");
            }

            if ((props["leaguename"] == null) || (props["leaguename"].Trim().Length == 0))
            {
                throw new ArgumentException("You must specify a league name.");
            }

            if (props["leaguename"].Trim().Length > cLeagueNameLength)
            {
                throw new ArgumentException(
                    "The league name cannot be longer than " + cLeagueNameLength + " characters.");
            }
        }

        /// <summary>
        /// Creates a collection with team names and points for all
        /// the points params.
        /// </summary>
        private NameValueCollection GetPointParams(NameValueCollection props)
        {
            NameValueCollection pointParams = new NameValueCollection();
            foreach (string paramName in props.AllKeys)
            {
                if (paramName.StartsWith("points"))
                {
                    string[] arr = props[paramName].Split(';');
                    pointParams.Add(arr[0], arr[1]);
                }
            }

            return pointParams;
        }

        /// <summary>
        /// Loads data for a league.
        /// </summary>
        public void LoadData(NameValueCollection props)
        {
            // This call will throw an ArgumentException if the
            // params aren't ok.
            ValidateParams(props);

            LeagueSKVFile matchFile = new LeagueSKVFile(props["matchfile"]);
            mArrTeams   = matchFile.LoadTeams(null);
            mArrMatches = matchFile.LoadMatches(null);
            NameValueCollection pointParams = GetPointParams(props);
                
            // Bail out if number of teams isn't ok
            if (mArrTeams.Count < 2 || mArrTeams.Count > 32)
            {
                throw new ArgumentException(
                    "The league can contain between 2 and 32 teams. " +
                    "This league contains " + mArrTeams.Count + " teams.");
            }

            // Loop over all the teams
            foreach (Team team in mArrTeams)
            {
                // Check the length of the team names and generate warnings
                if (team.Name.Length > cTeamNameLength)
                {
                    mWarningMessage += 
                        (team.Name + " will be truncated to " + 
                        team.Name.Substring(0, cTeamNameLength) + "|");
                }

                // Check if this team has a points parameter
                foreach (string teamName in pointParams.AllKeys)
                {
                    if (teamName == team.Name)
                    {
                        team.BonusPoints = StringToInt(pointParams[teamName], 0);
                    }
                }
            }

            // Create the league and set properties
            mLeague      = new League();
            mLeague.Name = props["leaguename"];
            mLeague.WinPoints  = StringToInt(props["winpoints"], 3);
            mLeague.DrawPoints = StringToInt(props["drawpoints"], 1);
            mLeague.LossPoints = StringToInt(props["losspoints"], 0);

            mLeague.TableLines = 0;
            int line = StringToInt(props["line1"], 0);
            if (line > 0 && line <= mArrTeams.Count)
                mLeague.TableLines |= (long)Math.Pow(2, line-1);
            line = StringToInt(props["line2"], 0);
            if (line > 0 && line <= mArrTeams.Count) 
                mLeague.TableLines |= (long)Math.Pow(2, line-1);
            line = StringToInt(props["line3"], 0);
            if (line > 0 && line <= mArrTeams.Count) 
                mLeague.TableLines |= (long)Math.Pow(2, line-1);
            line = StringToInt(props["line4"], 0);
            if (line > 0 && line <= mArrTeams.Count) 
                mLeague.TableLines |= (long)Math.Pow(2, line-1);

            mLeague.TableSort  = (TableSortMethod)StringToInt(props["tablesort"], 0);

            mLeague.TeamCount  = mArrTeams.Count;
            mLeague.MatchCount = mArrMatches.Count;

            mLeague.PlayedMatchCount = 0;
            mLeague.RoundCount = 1;
            int minYear = 9999;
            int maxYear = 0;
            // Loop over the matches to find the season, number of rounds and
            // number of played matches
            foreach (Match match in mArrMatches)
            {
                if (match.Round > mLeague.RoundCount) mLeague.RoundCount = match.Round;
                if (match.Date.Year < minYear)        minYear = match.Date.Year;
                if (match.Date.Year > maxYear)        maxYear = match.Date.Year;
                if (match.IsPlayed)                   mLeague.PlayedMatchCount++;
            }
            mLeague.Year    = minYear;
            mLeague.IsSplit = (minYear != maxYear);
        }

        /// <summary>
        /// Returns the default filename for the current league.
        /// </summary>
        public string GetDefaultFilename()
        {
            string filename = null;
            if (mLeague != null)
            {
                string fullFilename = 
                    LeagueBinaryFile.CreateFullFilename(mLeague.Name, mLeague.Year, mLeague.IsSplit);
                filename = Path.GetFileName(fullFilename);
            }
            return filename;
        }

        /// <summary>
        /// Saves the file using the default filename.
        /// </summary>
        public void Save()
        {
            Save(GetDefaultFilename());
        }

        /// <summary>
        /// Saves the file using the specified filename.
        /// </summary>
        public void Save(string filename)
        {
            LeagueBinaryFile outFile = new LeagueBinaryFile(filename);
            outFile.SaveHeader(mLeague);
            outFile.SaveTeams(mLeague, mArrTeams);
            outFile.SaveMatches(mLeague, mArrMatches);
        }

    }
}
