/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.0.0 July 2004
 * Copyright (c) 2004 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;
using System.Collections;
using System.IO;
using System.Reflection;
using System.Text;
using MBovin.TeamStats.Util;

namespace MBovin.TeamStats.Core
{
	/// <summary>
	/// Represents a binary file that stores league data.
	/// </summary>
	public class LeagueBinaryFile : ILeagueStore
	{
        // Constants for the file format
        private const int cFileVersion     = 1;
        private const int cHeaderSize      = 89;
        private const int cTeamSize        = 42;
        private const int cMatchSize       = 9;
        private const int cLeagueNameChars = 16;
        private const int cTeamNameChars   = 10;

        // The file name
        private string mFilename;


        /// <summary>
        /// Contructor.
        /// </summary>
        public LeagueBinaryFile(string filename)
		{
            mFilename = filename;
        }

        #region Properties

        /// <summary>
        /// The complete filename for this league file.
        /// </summary>
        public string Filename
        {
            get { return mFilename; }
            set { mFilename = value; }
        }

        #endregion

        #region ILeagueStore Members

        /// <summary>
        /// Loads the header record from the file. The data is
        /// stored in the supplied league object.
        /// </summary>
        public void LoadHeader(League league)
        {
            BinaryReader reader = new BinaryReader(File.OpenRead(mFilename));

            try 
            {
                // Check that it's a valid TeamStats file
                string idStr = new String(reader.ReadChars(4));
                if (idStr != "mbTS")
                {
                    reader.Close();
                    throw new System.IO.IOException("Not a TeamStats file.");
                }

                // Get the version of the file (not currently used)
                int fileVersion = reader.ReadByte();

                league.Name = (new UTF8Encoding().GetString(
                    reader.ReadBytes(cLeagueNameChars*4), 0, cLeagueNameChars*4)).TrimEnd('\0');
                int year    = (int)reader.ReadInt16();
                // Year is stored as a negative if season is split
                league.IsSplit = (year < 0);
                league.Year    = Math.Abs(year);

                league.TeamCount        = (int)reader.ReadByte();
                league.RoundCount       = (int)reader.ReadByte();
                league.MatchCount       = (int)reader.ReadInt16();
                league.PlayedMatchCount = (int)reader.ReadInt16();

                league.WinPoints  = (int)reader.ReadByte();
                league.DrawPoints = (int)reader.ReadByte();
                league.LossPoints = (int)reader.ReadByte();

                league.TableSort  = (TableSortMethod)reader.ReadByte();
                league.TableLines = reader.ReadInt64();
            }
            finally
            {
                if (reader != null) reader.Close();
            }
        }

        /// <summary>
        /// Returns a list of teams for the supplied league.
        /// </summary>
        public ArrayList LoadTeams(League league)
        {
            ArrayList listTeams;
            BinaryReader reader = new BinaryReader(File.OpenRead(mFilename));
            try
            {
                // Skip the header
                reader.ReadBytes(cHeaderSize);

                listTeams = new ArrayList(league.TeamCount);
                for (int i = 0; i < league.TeamCount; i++)
                {
                    listTeams.Add(LoadTeam(league, i, reader));
                }
            }
            finally
            {
                if (reader != null) reader.Close();
            }

            return listTeams;
        }

        /// <summary>
        /// Returns a list of the matches for the supplied league.
        /// </summary>
        public ArrayList LoadMatches(League league)
        {
            return LoadMatches(league, null);
        }

        /// <summary>
        /// Returns a list of matches. The delegate filterMethod is used
        /// to deterrmine which matches are included in the list.
        /// </summary>
        public ArrayList LoadMatches(League league, MatchFilter filterMethod, params object[] args)
        {
            ArrayList    matchList = null;
            BinaryReader reader = new BinaryReader(File.OpenRead(mFilename));

            try
            {
                // Skip the header and team section
                reader.ReadBytes(cHeaderSize + (league.TeamCount*cTeamSize));

                matchList = new ArrayList(league.MatchCount);
                for (int i = 0; i < league.MatchCount; i++)
                {
                    Match match = LoadMatch(league, i, reader);
                    if ((filterMethod == null) || filterMethod(match, args))
                    {
                        matchList.Add(match);
                    }
                }
            }
            finally
            {
                if (reader != null) reader.Close();
            }

            return matchList;
        }

        /// <summary>
        /// Saves the header record of the league.
        /// </summary>
        public void SaveHeader(League league)
        {
            BinaryWriter writer = new BinaryWriter(File.OpenWrite(mFilename));

            try 
            {
                byte[] idBytes = { (byte)'m', (byte)'b', (byte)'T', (byte)'S' } ;
                writer.Write(idBytes);
                writer.Write(Convert.ToByte(cFileVersion));
    
                writer.Write(Utils.GetUtf8Bytes(league.Name, cLeagueNameChars));
                writer.Write(Convert.ToInt16((league.IsSplit) ? -league.Year : league.Year));

                writer.Write(Convert.ToByte(league.TeamCount));
                writer.Write(Convert.ToByte(league.RoundCount));
                writer.Write(Convert.ToInt16(league.MatchCount));
                writer.Write(Convert.ToInt16(league.PlayedMatchCount));

                writer.Write(Convert.ToByte(league.WinPoints));
                writer.Write(Convert.ToByte(league.DrawPoints));
                writer.Write(Convert.ToByte(league.LossPoints));

                writer.Write(Convert.ToByte((int)league.TableSort));
                writer.Write(Convert.ToInt64(league.TableLines));
            }
            finally 
            {
                if (writer != null) writer.Close();
            }
        }

        /// <summary>
        /// Saves all the teams in the supplied list.
        /// </summary>
        public void SaveTeams(League league, ArrayList listTeams)
        {
            BinaryWriter writer = new BinaryWriter(File.OpenWrite(mFilename));

            try
            {
                writer.Seek(cHeaderSize, SeekOrigin.Begin); // Skip the header

                if (listTeams != null)
                {
                    foreach (Team team in listTeams)
                    {
                        SaveTeam(league, team, writer);
                    }
                }
            }
            finally 
            {
                if (writer != null) writer.Close();
            }
        }

        /// <summary>
        /// Saves all the teams in the supplied list.
        /// </summary>
        public void SaveMatches(League league, ArrayList listMatches)
        {
            BinaryWriter writer = new BinaryWriter(File.OpenWrite(mFilename));

            try
            {
                // Skip to first match
                int filePos = cHeaderSize + (league.TeamCount * cTeamSize);
                writer.Seek(filePos, SeekOrigin.Begin);

                if (listMatches != null)
                {
                    foreach (Match match in listMatches)
                    {
                        SaveMatch(league, match, writer);
                    }
                }
                // todo: filslut om antalet matcher minskat
            }
            finally 
            {
                if (writer != null) writer.Close();
            }
        }

        /// <summary>
        /// Saves the supplied match.
        /// </summary>
        public void SaveMatch(League league, Match match)
        {
            // Make sure that index is set
            if (match.Index < 0)
            {
                throw new InvalidOperationException("Index not set");
            }

            BinaryWriter writer = new BinaryWriter(File.OpenWrite(mFilename));
            try
            {
                // Skip to correct postion in the file
                int filePos = cHeaderSize + (league.TeamCount * cTeamSize) + (match.Index * cMatchSize);
                writer.Seek(filePos, SeekOrigin.Begin);
                SaveMatch(league, match, writer);
            }
            finally
            {
                if (writer != null) writer.Close();
            }
        }

        /// <summary>
        /// Saves the supplied team.
        /// </summary>
        public void SaveTeam(League league, Team team)
        {
            BinaryWriter writer = new BinaryWriter(File.OpenWrite(mFilename));
            
            try 
            {
                // Skip to correct postion in the file
                int filePos = cHeaderSize + (team.Index * cTeamSize);
                writer.Seek(filePos, SeekOrigin.Begin);
                SaveTeam(league, team, writer);
            }
            finally
            {
                if (writer != null) writer.Close();
            }
        }

        #endregion
        
        /// <summary>
        /// Loads a team.
        /// </summary>
        /// <param name="league"></param>
        /// <param name="index">The index of the the team</param>
        /// <param name="reader">The reader to load team from</param>
        /// <returns>A team object</returns>
        private Team LoadTeam(League league, int index, BinaryReader reader)
        {
            Team team = new Team(league, index);

            team.Name = (new UTF8Encoding().GetString(
                reader.ReadBytes(cTeamNameChars * 4), 0, cTeamNameChars * 4)).TrimEnd('\0');
            team.BonusPoints  = (int)reader.ReadSByte();
            team.HiddenPoints = (int)reader.ReadSByte();

            return team;
        }

        /// <summary>
        /// Loads a match.
        /// </summary>
        /// <param name="league"></param>
        /// <param name="index">The index of the match</param>
        /// <param name="reader">The reader to load match from</param>
        /// <returns>A match object</returns>
        private Match LoadMatch(League league, int index, BinaryReader reader)
        {
            Match match = new Match(league, index);

            match.Round      = (int)reader.ReadByte();
            int year         = (int)reader.ReadInt16();
            int month        = (int)reader.ReadByte();
            int day          = (int)reader.ReadByte();
            match.HomeTeamId = (int)reader.ReadByte();
            match.AwayTeamId = (int)reader.ReadByte();
            match.HomeGoals  = (int)reader.ReadSByte();
            match.AwayGoals  = (int)reader.ReadSByte();
            match.Date       = new DateTime(year, month, day);

            return match;
        }

        /// <summary>
        /// Saves a team.
        /// </summary>
        private void SaveTeam(League league, Team team, BinaryWriter writer)
        {
            writer.Write(Utils.GetUtf8Bytes(team.Name, cTeamNameChars));
            writer.Write(Convert.ToSByte(team.BonusPoints));
            writer.Write(Convert.ToSByte(team.HiddenPoints));
        }

        /// <summary>
        /// Saves a match.
        /// </summary>
        private void SaveMatch(League league, Match match, BinaryWriter writer)
        {
            writer.Write(Convert.ToByte(match.Round));
            writer.Write(Convert.ToInt16(match.Date.Year));
            writer.Write(Convert.ToByte(match.Date.Month));
            writer.Write(Convert.ToByte(match.Date.Day));
            writer.Write(Convert.ToByte(match.HomeTeamId));
            writer.Write(Convert.ToByte(match.AwayTeamId));
            writer.Write(Convert.ToSByte(match.HomeGoals));
            writer.Write(Convert.ToSByte(match.AwayGoals));
        }

        /// <summary>
        /// This method renames a league file. If the new filename 
        /// isn't available an IOException is thrown.
        /// </summary>
        public void Rename(string newFilename)
        {
            if (newFilename != mFilename)
            {
                // Eject if filename isn't available
                if (!IsFilenameAvailable(newFilename))
                {
                    throw new System.IO.IOException("Filename not available or invalid.");
                }

                // Rename the file
                File.Move(mFilename, newFilename);
                // Store the new filename
                mFilename = newFilename;
            }
        }

        /// <summary>
        /// Returns a complete filename. The filename is constructed
        /// from league name, year, and isSplit and the directory
        /// is the same as the executing program is in.
        /// </summary>
        public static string CreateFullFilename(string leaguename, int year, bool isSplit)
        {
            string seasonStr = (year % 100).ToString("00");
            if (isSplit) 
            {
                seasonStr += ((year+1) % 100).ToString("00");
            }

            string filename = leaguename + seasonStr + ".ts";
            string appDirectory = Path.GetDirectoryName(
                Assembly.GetExecutingAssembly().GetName().CodeBase);

            return Path.Combine(appDirectory, filename);
        }

        /// <summary>
        /// Checks if the specified filename is available, and doesn't
        /// contain any invalid characters.
        /// </summary>
        public static bool IsFilenameAvailable(string filename)
        {
            bool fileExists = File.Exists(filename);
            bool validName  = true;

            if (!fileExists)
            {
                try  
                {
                    FileStream fileStream = File.OpenWrite(filename);
                    fileStream.Close();
                    File.Delete(filename);
                }
                catch (ArgumentException) 
                {
                    validName = false;
                }
            }

            return (!fileExists && validName);
        }


    }
}
