/*
 * TeamStats by Mats Bovin (tsppc@mbovin.com)
 * v1.0.0 July 2004
 * Copyright (c) 2004 Mats Bovin
 *
 * This file is part of TeamStats.
 *
 * TeamStats is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * TeamStats is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with TeamStats; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

using System;


namespace MBovin.TeamStats.Core
{
	/// <summary>
	/// Implements a Match.
	/// </summary>
	public class Match : IComparable
	{
        private League mLeague;

        private int mIndex;
        private int mHomeTeamId;
        private int mAwayTeamId;
        private int mHomeGoals;
        private int mAwayGoals;
        private int mRound;
        private DateTime mDate;


        /// <summary>
        /// Contructor.
        /// </summary>
        public Match(League league, int index)
        {
            mLeague = league;
            mIndex  = index;
        }

        /// <summary>
        /// Constructor.
        /// </summary>
        public Match(League league, int round, DateTime date, int homeTeamId, int awayTeamId)
        {
            mLeague     = league;
            mRound      = round;
            mDate       = date;
            mHomeTeamId = homeTeamId;
            mAwayTeamId = awayTeamId;
            mHomeGoals  = -1;
            mAwayGoals  = -1;
            mIndex      = -1;
        }

        #region Properties

        public int Index
        {
            get { return mIndex; }
            set { mIndex = value; }
        }

        public int HomeTeamId
        {
            get { return mHomeTeamId; }
            set { mHomeTeamId = value; }
        }

        public string HomeTeamName
        {
            get { return mLeague.GetTeam(mHomeTeamId).Name; }
        }

        public int AwayTeamId
        {
            get { return mAwayTeamId; }
            set { mAwayTeamId = value; }
        }

        public string AwayTeamName
        {
            get { return mLeague.GetTeam(mAwayTeamId).Name; }
        }

        public int HomeGoals
        {
            get { return mHomeGoals; }
            set { mHomeGoals = value; }
        }

        public string HomeGoalsString
        {
            get { return (this.IsPlayed) ? mHomeGoals.ToString() : ""; }
        }

        public int AwayGoals
        {
            get { return mAwayGoals; }
            set { mAwayGoals = value; }
        }

        public string AwayGoalsString
        {
            get { return (this.IsPlayed) ? mAwayGoals.ToString() : ""; }
        }

        public int Round
        {
            get { return mRound; }
            set { mRound = value; }
        }

        public DateTime Date
        {
            get { return mDate; }
            set { mDate = value; }
        }

        public bool IsPlayed 
        {
            get { return mHomeGoals > -1; }
        }

        #endregion


        /// <summary>
        /// Returns the opponent of the specified team. If the team didn't
        /// play this match null is returned.
        /// </summary>
        public Team GetOpponent(Team team)
        {
            if (team.Index == mHomeTeamId) 
                return mLeague.GetTeam(mAwayTeamId);
            else if (team.Index == mAwayTeamId) 
                return mLeague.GetTeam(mHomeTeamId);
            else 
                return null;
        }

        /// <summary>
        /// Returns the number of goals scored by the specified team. 
        /// Returns -1 if the team didn't play this match.
        /// </summary>
        public int GetGoalsFor(Team team)
        {
            if (team.Index == mHomeTeamId) 
                return mHomeGoals;
            else if (team.Index == mAwayTeamId) 
                return mAwayGoals;
            else 
                return -1;
        }

        /// <summary>
        /// Returns the number of goals conceded by the specified team. 
        /// Returns -1 if the team didn't play this match.
        /// </summary>
        public int GetGoalsAgainst(Team team)
        {
            if (team.Index == mHomeTeamId) 
                return mAwayGoals;
            else if (team.Index == mAwayTeamId) 
                return mHomeGoals;
            else 
                return -1;
        }


        /// <summary>
        /// Sets the result of this match.
        /// </summary>
        public void SetResult(int homeGoals, int awayGoals)
        {
            if ((homeGoals < 0) || (awayGoals < 0))
            {
                SetNotPlayed();
            }
            else
            {
                mHomeGoals = homeGoals;
                mAwayGoals = awayGoals;
            }
        }

        /// <summary>
        /// Sets this match to "not yet played".
        /// </summary>
        public void SetNotPlayed()
        {
            mHomeGoals = -1;
            mAwayGoals = -1;
        }

        /// <summary>
        /// Saves the match data.
        /// </summary>
        public void Save()
        {
            mLeague.LeagueStore.SaveMatch(mLeague, this);
        }


        /// <summary>
        /// Returns a string representation of this Match.
        /// </summary>
        public override string ToString()
        {
            return mDate.Day + "/" + mDate.Month + " " + 
                   HomeTeamName + "-" + AwayTeamName + " " +
                   HomeGoalsString + "-" + AwayGoalsString;
        }


        #region IComparable Members

        /// <summary>
        /// Compares this match another (by the match date).
        /// Used by sort methods.
        /// </summary>
        public int CompareTo(object obj)
        {
            Match otherMatch = (Match)obj;

            if (this.Date < otherMatch.Date) return -1;
            if (this.Date > otherMatch.Date) return  1;
            return 0;
        }

        #endregion
    }
}
